/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   info@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "icecentralwidget.h"

ICECentralWidget::ICECentralWidget(QWidget *parent)
 : QWidget(parent)
 , mSessionObject(NULL)
{
}


ICECentralWidget::~ICECentralWidget()
{
	if ( mSessionObject != NULL ) {
		destroySessionObject();
	}
}


/**
** Relase outstanding reference and destroy the session object.
*/
void ICECentralWidget::destroySessionObject()
{
	if ( mSessionObject != NULL ) {
		mSessionObject->release();	/** release initial reference to the object */
		mSessionObject->destroy();	/** queue for destruction */
		mSessionObject=NULL;
	}
}

/**
** Set the session object for this session object manager.
*/
void ICECentralWidget::setSessionObject(ICEClientSessionObjectUiTransport* so) 
{ 
	if ( ICESessionObject::obtain( so->id() ) != NULL ) { /** obtain an initial reference to the new object */
		destroySessionObject();						/** release initial reference to old object and destroy it */
		mSessionObject = so;		
		if ( mSessionObject->widget() != NULL ) {
			mSessionObject->widget()->setParent(this);
			mSessionObject->widget()->resize( size() );
			mSessionObject->widget()->show();
			mSessionObject->widget()->raise();
		}
	} 
}

void ICECentralWidget::resizeEvent(QResizeEvent* e)
{
	if ( mSessionObject != NULL && mSessionObject->widget() != NULL ) {
		mSessionObject->widget()->resize( e->size() );
	}
}
