/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   info@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the teice of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QtGui>

#include <assert.h>

#include "iceclient.h"
#include "icemd5.h"
#include "icesessionobject.h"

ICEClient::ICEClient()
	: mSlowTimer(-1)
	, mFastTimer(-1)
	, mTrayIcon(NULL)
{
	session()->setMainWindow(this);
	iconButtonExit			= QIcon( ":/icons/button_cancel.png" );
	iconButtonConfigure 	= QIcon( ":/icons/button_configure.png" );
	iconButtonSettings  	= QIcon( ":/icons/button_settings.png" );
	iconConnectNo 			= QIcon( ":/icons/connect_no.png" );
	iconConnectCreating 	= QIcon( ":/icons/connect_creating.png" );
	iconConnectEstablished 	= QIcon( ":/icons/connect_established.png" );

	iconLocationGo			= QIcon( ":/icons/location_goto.png" );
	iconLocationUp			= QIcon( ":/icons/location_up.png" );
	iconLocationBack		= QIcon( ":/icons/location_back.png" );
	iconLocationForward		= QIcon( ":/icons/location_forward.png" );
	iconLocationLast		= QIcon( ":/icons/location_last.png" );
	iconLocationHome		= QIcon( ":/icons/location_home.png" );
	
	loginForm = new Ui::ICELoginForm();
	loginDialog = new QDialog;
	loginForm->setupUi(loginDialog);

	mCentralWidget = new ICECentralWidget();
	setCentralWidget(mCentralWidget);

	QObject::connect(loginForm->newAccountButton,SIGNAL(clicked()),this,SLOT(newAccount()));
	QObject::connect(loginForm->deleteAccountButton,SIGNAL(clicked()),this,SLOT(deleteAccount()));
	QObject::connect(loginForm->changePasswordButton,SIGNAL(clicked()),this,SLOT(changePassword()));

	QObject::connect(session(),SIGNAL(statusChanged(ICEStatus::ConnectStatus)),status(),SLOT(setStatus(ICEStatus::ConnectStatus)));
	QObject::connect ( this,SIGNAL ( statusMessage ( void*,ICESession::MessageReason,QString ) ),
					   this,SLOT ( slotStatusMessage ( void*,ICESession::MessageReason,QString ) ) );
	QObject::connect ( session(),SIGNAL ( statusMessage ( void*,ICESession::MessageReason,QString ) ),
					   this,SLOT ( slotStatusMessage ( void*,ICESession::MessageReason,QString ) ) );
	QObject::connect(this,SIGNAL(go(const QString&)),session(),SLOT(go(const QString&)));
	
	createActions();
	createMenus();
	createToolBars();
	createStatusBar();
		
	mSlowTimer = startTimer(1000);
	mFastTimer = startTimer(50);

	readSettings();
	slotLocationHome();
}

void ICEClient::connectToServer()
{
	if ( loginDialog->exec() ) {
		writeSettings();
		session()->disconnectFromHost();
		session()->connectToHost();
	}
	//loginDialog->hide();
}

void ICEClient::disconnectFromServer()
{
	session()->disconnectFromHost();
}

void ICEClient::newAccount()
{
	/**
	 ** Force new MD5 sum to be produced...
	 **/
	writeSettings();
	readSettings();

	/**
	 ** Tell the main view...
	 */
	//view->slotSettingsChanged();
	session()->newAccount();
}

void ICEClient::deleteAccount()
{
	session()->deleteAccount();
}

void ICEClient::changePassword()
{

	session()->changePassword();
}

void ICEClient::closeEvent(QCloseEvent* e)
{
	writeSettings();
	delete loginForm;
	delete loginDialog;
	e->accept();
}

void ICEClient::timerEvent(QTimerEvent* e)
{
	if ( e->timerId() == mSlowTimer ) {
		bool sessionValidated = ( session()->state() == ICESession::SessionValidated );
		loginForm->newAccountButton->setEnabled(!sessionValidated);
		loginForm->deleteAccountButton->setEnabled(sessionValidated);
		loginForm->changePasswordButton->setEnabled(sessionValidated);
	} else if ( e->timerId() == mFastTimer ) {
		/**
		 ** Periodically purge session objects...
		 */
		ICESessionObject::purge();
	}
}

void ICEClient::tag()
{
}

void ICEClient::untag()
{
}

void ICEClient::restore()
{
}

void ICEClient::properties()
{
}

void ICEClient::configsettings()
{
	if ( loginDialog->exec() ) {
		fprintf( stderr, "settings accepted\n" );
		writeSettings();
	}
	loginDialog->hide();
}

void ICEClient::readSettings()
{
	QSettings settings(QSettings::NativeFormat, QSettings::UserScope, "pikeaero.com", "ICE Client");

	settings.beginGroup("MainWindow");
	QPoint pos = settings.value("pos", QPoint(200, 200)).toPoint();
	QSize size = settings.value("size", QSize(400, 400)).toSize();
	settings.endGroup();

	if ( loginForm != NULL ) {
		settings.beginGroup("Service");
		loginForm->server->setEditText(settings.value("hostname","ice.pikeaero.com").toString());
			//loginForm->hostport->setText(settings.value("hostport","8088").toString());
		loginForm->login->setText(settings.value("login","guest").toString());
		loginForm->password->setText(settings.value("password","2ec099f2d602cc4968c5267970be1326").toString()); 
		loginForm->email->setText(settings.value("email","").toString());
		loginForm->cipher0button->setChecked(settings.value("cipher0",false).toBool());
		loginForm->cipher1button->setChecked(settings.value("cipher1",true).toBool());
		loginForm->cipher2button->setChecked(settings.value("cipher2",false).toBool());
		loginForm->cipher3button->setChecked(settings.value("cipher3",false).toBool());
		loginForm->cipher4button->setChecked(settings.value("cipher4",false).toBool());
		loginForm->cipher5button->setChecked(settings.value("cipher5",false).toBool());
		loginForm->cipher6button->setChecked(settings.value("cipher6",false).toBool());
		loginForm->cipher7button->setChecked(settings.value("cipher7",false).toBool());
		loginForm->cipher8button->setChecked(settings.value("cipher8",false).toBool());
		loginForm->cipher9button->setChecked(settings.value("cipher9",false).toBool());
		loginForm->cipher10button->setChecked(settings.value("cipher10",false).toBool());
		loginForm->log_enabled->setChecked(settings.value("log_enabled",true).toBool());
		settings.endGroup();
	}
	resize(size);
	move(pos);
	session()->setHost( loginForm->server->currentText(), 8088 );
	session()->setLogin( loginForm->login->text() );
	session()->setKey( loginForm->password->text() );
	
	settings.beginGroup("location");
		home = settings.value("home", "ice://webice.org").toString();
	settings.endGroup();

	if ( loginForm->cipher0button->isChecked() ) {
		session()->setCipher( ICEPacket::CipherXOR );
	} else if ( loginForm->cipher1button->isChecked() ) {
		session()->setCipher( ICEPacket::CipherBlowfish );
	} else if ( loginForm->cipher2button->isChecked() ) {
		session()->setCipher( ICEPacket::CipherSEAL );
	} else if ( loginForm->cipher3button->isChecked() ) {
		session()->setCipher( ICEPacket::CipherLOKI91 );
	} else {
		fprintf( stderr, "cipher not supported\n" );
		assert(0);
	}
	
}

void ICEClient::writeSettings()
{
	QSettings settings(QSettings::NativeFormat, QSettings::UserScope, "pikeaero.com", "ICE Client");

	settings.beginGroup("MainWindow");
	settings.setValue("pos", pos());
	settings.setValue("size", size());
	settings.endGroup();


	settings.beginGroup("Service");
	settings.setValue("hostname",loginForm->server->currentText());
	settings.setValue("hostport",(int)8088);
	settings.setValue("login",loginForm->login->text());
	if ( loginForm->password->text() != settings.value("password","2ec099f2d602cc4968c5267970be1326").toString() ) {
		ICEMD5 md5(loginForm->password->text());
		settings.setValue("password",md5.digest());
	}
	settings.setValue("email",loginForm->email->text());
	settings.setValue("cipher0",loginForm->cipher0button->isChecked());
	settings.setValue("cipher1",loginForm->cipher1button->isChecked());
	settings.setValue("cipher2",loginForm->cipher2button->isChecked());
	settings.setValue("cipher3",loginForm->cipher3button->isChecked());
	settings.setValue("cipher4",loginForm->cipher4button->isChecked());
	settings.setValue("cipher5",loginForm->cipher5button->isChecked());
	settings.setValue("cipher6",loginForm->cipher6button->isChecked());
	settings.setValue("cipher7",loginForm->cipher7button->isChecked());
	settings.setValue("cipher8",loginForm->cipher8button->isChecked());
	settings.setValue("cipher9",loginForm->cipher9button->isChecked());
	settings.setValue("cipher10",loginForm->cipher10button->isChecked());
	settings.setValue("log_enabled",loginForm->log_enabled->isChecked());
	settings.endGroup();
	
	settings.beginGroup("location");
		settings.setValue("home",home);
	settings.endGroup();
	
	emit settingsChanged();
	readSettings();
}

void ICEClient::about()
{
	QMessageBox::about( this, QString("About ") + trademark(),
						QString( trademark() + " Version " + version() + "\n"
								+ copyright()+"\n"
								"All rights reserved.\n\n\n"
								"Mike Sharkey <mike@pikeaero.com>\n"
								"ice://guest:guest@pikeaero.com\n"
								"http://pikeaero.com/ice\n"
								"\n\n"
								"This program is free software; you can redistribute it and/or modify "
								"it under the teice of the GNU General Public License as published by "
								" the Free Software Foundation; either version 2 of the License, or "
								"(at your option) any later version.\n"
								"\n"
								"This program is distributed in the hope that it will be useful, "
								"but WITHOUT ANY WARRANTY; without even the implied warranty of "
								" MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
								"GNU General Public License for more details.\n"
								"\n"
								"You should have received a copy of the GNU General Public License "
								"along with this program; if not, write to the "
								"Free Software Foundation, Inc.,\n"
								"59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.\n"
								"\n"));
}

void ICEClient::createActions()
{
	connectAct = new QAction(iconConnectEstablished, tr("&Connect"), this);
	connectAct->setShortcut(tr("Ctrl+Y"));
	connectAct->setStatusTip(tr("Connect to Server"));
	connect(connectAct, SIGNAL(triggered()), this, SLOT(connectToServer()));
	
	disconnectAct = new QAction(iconConnectNo, tr("&Disconnect"), this);
	disconnectAct->setShortcut(tr("Ctrl+K"));
	disconnectAct->setStatusTip("Disconnect from "+trademark()+" Messaging Server");
	connect(disconnectAct, SIGNAL(triggered()), this, SLOT(disconnectFromServer()));
	
	exitAct = new QAction(iconButtonExit,tr("E&xit"), this);
	exitAct->setShortcut(tr("Ctrl+Q"));
	exitAct->setStatusTip(tr("Exit the application"));
	connect(exitAct, SIGNAL(triggered()), this, SLOT(close()));

	aboutAct = new QAction(tr("&About"), this);
	aboutAct->setStatusTip(tr("Show the application's About box"));
	connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));


	/**
	 ** location bar actions...
	 **/
	locationGo = new QAction(iconLocationGo, tr("Go"), this);
	locationGo->setShortcut(tr("Ctrl+M"));
	locationGo->setStatusTip(tr("Go to the requested location."));
	connect(locationGo, SIGNAL(triggered()), this, SLOT(slotLocationGo()));
	
	locationUp = new QAction(iconLocationUp, tr("Up"), this);
	locationUp->setStatusTip(tr("Go up."));
	connect(locationUp, SIGNAL(triggered()), this, SLOT(slotLocationUp()));
	
	locationBack = new QAction(iconLocationBack, tr("Back"), this);
	locationBack->setStatusTip(tr("Go Back."));
	connect(locationBack, SIGNAL(triggered()), this, SLOT(slotLocationBack()));
	
	locationForward = new QAction(iconLocationForward, tr("Forward"), this);
	locationForward->setStatusTip(tr("Go forward."));
	connect(locationForward, SIGNAL(triggered()), this, SLOT(slotLocationForward()));
	
	locationLast = new QAction(iconLocationLast, tr("Last"), this);
	locationLast->setStatusTip(tr("Go to last."));
	connect(locationLast, SIGNAL(triggered()), this, SLOT(slotLocationLast()));
	
	locationHome = new QAction(iconLocationHome, tr("Home"), this);
	locationHome->setStatusTip(tr("Go Home."));
	connect(locationHome, SIGNAL(triggered()), this, SLOT(slotLocationHome()));
		
}

void ICEClient::createMenus()
{
	fileMenu = menuBar()->addMenu(tr("&File"));
	fileMenu->addAction(exitAct);

	locationMenu = menuBar()->addMenu(tr("&Location"));
	locationMenu->addAction(locationGo);		/** goto location in location bar */
	locationMenu->addAction(locationUp);		/** go up in folder herarchy in location bar */
	locationMenu->addAction(locationBack);		/** go back */
	locationMenu->addAction(locationForward);	/** go forward */
	locationMenu->addAction(locationLast);		/** goto last location */
	locationMenu->addAction(locationHome);		/** goto homelocation */
	
	serverMenu = menuBar()->addMenu(tr("&Server"));
	serverMenu->addAction(connectAct);
	serverMenu->addAction(disconnectAct);
	
	menuBar()->addSeparator();
	
	helpMenu = menuBar()->addMenu(tr("&Help"));
	helpMenu->addAction(aboutAct);
		
}

void ICEClient::createToolBars()
{
	locationToolBar = addToolBar(tr("&Location"));
	locationToolBar->addAction(locationUp);		/** go up in folder herarchy in location bar */
	locationToolBar->addAction(locationBack);		/** go back */
	locationToolBar->addAction(locationForward);	/** go forward */
	locationToolBar->addAction(locationLast);		/** goto last location */
	locationToolBar->addAction(locationHome);		/** goto homelocation */
	
	locationEditor = new QComboBox;
	locationEditor->setMinimumWidth(400);
	locationToolBar->addWidget( locationEditor );
	QObject::connect(locationEditor,SIGNAL(activated(const QString&)),this,SLOT(slotLocationGo(const QString&)));
	locationEditor->setEditable(true);
	locationEditor->setInsertPolicy(QComboBox::InsertAtTop);
			
	locationToolBar->addAction(locationGo);		/** goto location in location bar */
	
}

void ICEClient::createStatusBar()
{
	statusBar()->addPermanentWidget(status());
	statusBar()->showMessage(tr("Ready"));
}

QString ICEClient::strippedName(const QString &fullFileName)
{
	return QFileInfo(fullFileName).fileName();
}

void ICEClient::statusChanged(QString msg)
{
	QString log_msg = QDateTime::currentDateTime().toString(Qt::ISODate) + 
			QString("@") + 
			session()->hostname() +
			QString(": ") +
			msg;
	statusBar()->showMessage(msg);
	if ( loginForm->log_enabled->isChecked() ) {
		loginForm->logBrowser->append( log_msg );
	}
	if ( msg[0] == '*' ) {
		QMessageBox::information(this,trademark() + " Status",log_msg,QMessageBox::Ok);
	}
}


void ICEClient::slotStatusMessage ( void* session, ICESession::MessageReason reason, QString msg )
{
	char buf[128];
	
	/** Format the session address... */
	switch(sizeof(char*)) {
		case 3:
			case 4:		sprintf( buf, "%08p",  (char*)session ); break;
			case 8:		sprintf( buf, "%016p", (char*)session ); break;
			case 16:	sprintf( buf, "%032p", (char*)session ); break;
			default:	sprintf( buf, "%p",     (char*)session ); break;
	}
	QString reason_str;
	switch ( reason )
	{
		default:
			case ICESession::ReasonUnknown:   reason_str = "UNKNOWN  "; break;
			case ICESession::ReasonVersion:   reason_str = "VERSION  "; break;
			case ICESession::ReasonServer:    reason_str = "SERVER   "; break;
			case ICESession::ReasonSession:   reason_str = "SESSION  "; break;
			case ICESession::ReasonSessionObj:reason_str = "SESS_OBJ "; break;
			case ICESession::ReasonDirectory: reason_str = "DIRECTORY"; break;
			case ICESession::ReasonDatabase:  reason_str = "DATABASE "; break;
			case ICESession::ReasonDebug:     reason_str = "DEBUG    "; break;
			case ICESession::ReasonDebugTX:   reason_str = "DEBUG_TX "; break;
			case ICESession::ReasonDebugRX:   reason_str = "DEBUG_RX "; break;
	}
	QString log_msg = QDateTime::currentDateTime().toString ( Qt::ISODate ) + QString ( "|" ) +
			reason_str + QString ( "|" ) +
			QString ( buf ) + QString ( "| " ) +
			msg;

	printf ( "%s\n", log_msg.toLatin1().data() );
}


void ICEClient::slotLocationGo()
{
	QString location = locationEditor->currentText();
	slotLocationGo( location );
}

/**
** Go to a location URL. If the URL is not in the location editor list already, then add it.
** If the location list exceeds maximum number of items (FIXME - make it a settings option, currently fixed at 30).
*/
void ICEClient::slotLocationGo(const QString& location)
{
	if ( location != locationEditor->currentText() ) {
		locationEditor->setEditText( location );
		if ( locationEditor->findText( location ) < 0 ) {
			locationEditor->addItem( location );
		}
		while( locationEditor->count() > 30 ) {
			locationEditor->removeItem(0);
		}
	}
	emit go(location);
}

void ICEClient::slotLocationUp()
{
}

void ICEClient::slotLocationBack()
{	
}

void ICEClient::slotLocationForward()
{
}

void ICEClient::slotLocationLast()
{
}

void ICEClient::slotLocationHome()
{
	slotLocationGo( home );
}

