/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   info@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the teice of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ICECLIENT_H
#define ICECLIENT_H

#include <QMainWindow>
#include <QIcon>
#include <QSettings>
#include <QSystemTrayIcon>
#include <QComboBox>

#include "icecentralwidget.h"

class QAction;
class QMenu;

#include "icestatus.h"
#include "iceclientsession.h"
#include "ui_iceloginform.h"
				 

class ICEClient : public QMainWindow
{
	Q_OBJECT

	public:
		ICEClient();
		static QString trademark() {return QString("ICE Client") + '®';}
		static QString copyright() {return QString( QString("Copyright") + '©' ) + "2007 Pike Aerospace Research Corp.";}
		static QString version()   {return "0.3.5";}
	public slots:
		void slotStatusMessage ( void* session, ICESession::MessageReason, QString msg );
	
	signals:
		void statusMessage ( void *session, ICESession::MessageReason, QString msg );
		void settingsChanged();
		void go(const QString&);

	protected:
		void closeEvent(QCloseEvent* e);
		void timerEvent(QTimerEvent* e);

	private slots:
		void connectToServer();
		void disconnectFromServer();
		void tag();
		void untag();
		void restore();
		void properties();
		void configsettings();
		void about();
		void statusChanged(QString msg);

		void newAccount();
		void deleteAccount();
		void changePassword();

		/**
		 ** Location bar handlers...
		 **/
		void		slotLocationGo(const QString& location);
		void		slotLocationGo();
		void		slotLocationUp();
		void		slotLocationBack();
		void		slotLocationForward();
		void		slotLocationLast();
		void		slotLocationHome();
	
	private:
		inline ICEClientSession*	session() {return &mSession;}
		inline ICEStatus*			status() {return &mStatus;}
		void createActions();
		void createMenus();
		void createToolBars();
		void createStatusBar();
		void readSettings();
		void writeSettings();
		QString strippedName(const QString &fullFileName);

		QMenu* fileMenu;
		QMenu* serverMenu;
		QMenu* helpMenu;
		
		QToolBar* fileToolBar;
		QToolBar* serverToolBar;
		
		QAction *connectAct;
		QAction *disconnectAct;
		QAction *exitAct;
		QAction *aboutAct;

		QIcon iconButtonExit;
		QIcon iconButtonConfigure;
		QIcon iconButtonSettings;
		QIcon iconConnectNo;
		QIcon iconConnectCreating;
		QIcon iconConnectEstablished;

		Ui::ICELoginForm*    loginForm;
		QDialog*             loginDialog;
		
		int    mSlowTimer;
		int    mFastTimer;
		
		ICEStatus			mStatus;
		ICEClientSession	mSession;
		QSystemTrayIcon*	mTrayIcon;
		ICECentralWidget*	mCentralWidget;
		
		/**
		 ** Items related to the location bar..
		 */
		QString		home;				/** home location */
		QComboBox*	locationEditor;		/** location editor */
		QMenu*		locationMenu;
		QToolBar*	locationToolBar;
		
		QAction*	locationGo;			/** goto location in location bar */
		QAction*	locationUp;			/** go up in folder herarchy in location bar */
		QAction*	locationBack;		/** go back */
		QAction*	locationForward;	/** go forward */
		QAction*	locationLast;		/** goto last location */
		QAction*	locationHome;		/** goto homelocation */
		
		QIcon		iconLocationGo;
		QIcon		iconLocationUp;
		QIcon		iconLocationBack;
		QIcon		iconLocationForward;
		QIcon		iconLocationLast;
		QIcon		iconLocationHome;	
		
};

extern QMainWindow* mainWindow;

#endif
