/***************************************************************************
 *   Copyright (C) 2007 Pike Aerospace Research Corporation                *
 *   info@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the teice of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "iceclientsession.h"
#include "iceclientsessionobjectuitransport.h"
#include "iceclientsessionobjectuielementtransport.h"

#include <QApplication>
#include <QFileInfo>
#include <QMessageBox>

#define inherited ICESession

ICEClientSession::ICEClientSession ( QObject *parent )
: inherited ( parent,false )
, mRequestingNewID ( false )
, mDeletingAccount ( false )
{
	qRegisterMetaType<QAbstractSocket::SocketError>( "QAbstractSocket::SocketError" );
	qRegisterMetaType<QAbstractSocket::SocketState>( "QAbstractSocket::SocketState" );	
	setDebugging ( true );
	mSocket = new QTcpSocket;
	initialize();
}


ICEClientSession::~ICEClientSession()
{
}

void ICEClientSession::requestDisconnect ( ICEPacket::PacketCode code )
{
	if ( state() != SessionClosed && state() != SessionClosing )
	{
		ICEPacket txpacket;
		setState ( SessionClosing );
		txpacket.setType ( ICEPacket::RequestDisconnect );
		txpacket.setCode ( code );
		tx ( txpacket );
		setState ( SessionClosing );
		QTimer::singleShot ( 3000,this,SLOT ( close() ) );
	}
	if ( code != ICEPacket::CodeOK )
	{
		emit packetError ( code,"" );
	}
}

void ICEClientSession::requestSession ( ICEPacket* rxpacket )
{
	if ( state() == SessionChallenge )
	{
		QByteArray cipher_challenge;
		QByteArray challenge;
		ICEPacket txpacket;
		txpacket.setType ( ICEPacket::RequestSession );
		txpacket.setCode ( ICEPacket::CodeOK );
		challenge = rxpacket->getData();
		if ( mCryptRX != NULL )
		{
			mCryptRX->cipher ( false, getKey(), &challenge, &cipher_challenge );
		}
		txpacket.setData ( cipher_challenge );
		tx ( txpacket );
	}
	else
	{
		requestDisconnect ( ICEPacket::CodeProtocolError );
	}
}

void ICEClientSession::requestLogin ( ICEPacket* rxpacket )
{
	if ( state() == SessionConnected )
	{
		ICEPacket txpacket;
		if ( mCryptRX != NULL ) {delete mCryptRX; mCryptRX=NULL;}
		if ( mCryptTX != NULL ) {delete mCryptTX; mCryptTX=NULL;}
		switch ( getCipher() )
		{
			case ICEPacket::CipherSEAL:
				mCryptRX = new ICECryptSEAL();
				mCryptTX = new ICECryptSEAL();
				break;
			case ICEPacket::CipherBlowfish:
				mCryptRX = new ICECryptBF();
				mCryptTX = new ICECryptBF();
				break;
			case ICEPacket::CipherLOKI91:
				mCryptRX = new ICECryptLOKI91();
				mCryptTX = new ICECryptLOKI91();
				break;
			case ICEPacket::CipherDES:
			case ICEPacket::CipherRC5:
			case ICEPacket::Cipher3Way:
			case ICEPacket::CipherGOST:
			case ICEPacket::CipherIDEA:
			case ICEPacket::CipherA5:
			case ICEPacket::CipherPIKE:
			case ICEPacket::CipherXOR:
				mCryptRX = new ICECrypt();
				mCryptTX = new ICECrypt();
				break;
			default:
				requestDisconnect ( ICEPacket::CodeCipherNotSupported );
				return;
				break;
		}
		txpacket.setCipher ( getCipher() );
		txpacket.setType ( ICEPacket::RequestLogin );
		txpacket.setCode ( ICEPacket::CodeOK );
		txpacket.setLogin ( getLogin() );
		setEncrypt ( false );
		tx ( txpacket );
	}
	else
	{
		requestDisconnect ( ICEPacket::CodeProtocolError );
	}
	ICEPacket txpacket;
}

void ICEClientSession::requestConnection()
{
	if ( state() == SessionClosed )
	{
		ICEPacket txpacket;
		ICEPacket::PlatformType platform=ICEPacket::Unknown;
#if defined(Q_WS_WIN)
		platform = ICEPacket::Windows;
#elif defined(Q_WS_MACX)
		platform = ICEPacket::MacOSX;
#elif defined(Q_WS_QWS)
		platform = ICEPacket::Qtopia;
#elif defined(Q_WS_QWS)
		platform = ICEPacket::Unix;
#endif
		txpacket.setPlatform ( platform );
		txpacket.setType ( ICEPacket::RequestConnection );
		txpacket.setCode ( ICEPacket::CodeOK );
		txpacket.setLogin ( getLogin() );
		setEncrypt ( false );
		tx ( txpacket );
	}
	else
	{
		requestDisconnect ( ICEPacket::CodeProtocolError );
	}
}

void ICEClientSession::requestConnectNewLoginID()
{
	if ( state() == SessionClosed )
	{
		ICEPacket txpacket;
		txpacket.setType ( ICEPacket::RequestConnectNewLoginID );
		txpacket.setCode ( ICEPacket::CodeOK );
		txpacket.setLogin ( getLogin() );
		txpacket.setKey ( getKey() );
		setEncrypt ( false );
		tx ( txpacket );
	}
	else
	{
		requestDisconnect ( ICEPacket::CodeProtocolError );
	}
}

void ICEClientSession::requestDeleteLogin()
{
	if ( state() == SessionValidated )
	{
		ICEPacket txpacket;
		txpacket.setType ( ICEPacket::RequestDeleteLogin );
		txpacket.setCode ( ICEPacket::CodeOK );
		tx ( txpacket );
	}
	else
	{
		requestDisconnect ( ICEPacket::CodeProtocolError );
	}
}

ICESessionObject* ICEClientSession::construct(ICESessionObject::ObjectType type,QByteArray* id,QByteArray* data)
{
	ICESessionObject* so=NULL;
	bool dataIsLocation=false;
	switch( type ) {
		case ICESessionObject::TypeUiTransport:	
			dataIsLocation=true;		
			so = new ICEClientSessionObjectUiTransport(this,id); 
			break;
		case ICESessionObject::TypeUiElementTransport:	
			dataIsLocation=true;		
			so = new ICEClientSessionObjectUiElementTransport(this,id); 
			break;
		default:
			break; 
	}
	if ( so != NULL ) {
		ICESessionObject* soLock = ICESessionObject::obtain( so->id(), false );
		if ( soLock != NULL ) {
			if ( data != NULL ) {
				if ( dataIsLocation ) {
					so->setLocation(QString(*data));
				} else {
					so->setUi(*data);
				}
			}
			requestSessionObjectConstruct(so);
			soLock->release(false);
		}
	}
	return so; 	
}

void ICEClientSession::replySessionObjectConstruct( ICEPacket* rxpacket,ICEPacket::PacketCode code)
{
	ICESessionObject* so=NULL;
	QByteArray id = rxpacket->getObjectId();
	switch( (ICESessionObject::ObjectType)rxpacket->getObjectType() ) {
		case ICESessionObject::TypeUiTransport: 
			so = new ICEClientSessionObjectUiTransport(this,&id);
			break;
		case ICESessionObject::TypeUiElementTransport: 
			so = new ICEClientSessionObjectUiElementTransport(this,&id);
			break;
		default:
			break;
	}
	inherited::replySessionObjectConstruct( rxpacket, code );
}

void ICEClientSession::processReplyDisconnect(ICEPacket* rxpacket)
{
	setState(SessionClosing);
	QTimer::singleShot(3000,this,SLOT(close()));
}

void ICEClientSession::processReplySession(ICEPacket* rxpacket)
{
	if ( rxpacket->getCode() == ICEPacket::CodeOK ) {
		setState(SessionValidated);
		setEncrypt( true );
		go( mInitialLocation );
	} else {
		requestDisconnect();
	}
}

void ICEClientSession::processReplyLogin(ICEPacket* rxpacket)
{
	if ( rxpacket->getCode() == ICEPacket::CodeOK ) {
		setState(SessionChallenge);
		requestSession( rxpacket );
	} else {
		if ( rxpacket->getCode() == ICEPacket::CodeCipherNotSupported ) {
			/** Server does not support the cipher we tried,
			 ** lets try another.... */
			int cipherno = (int)mCipher;
			cipherno += 1;
			if ( cipherno >= (int)ICEPacket::CipherMAX )
				cipherno = 0;
			mCipher = (ICEPacket::PacketCipher)cipherno;
			rxpacket->setCode(ICEPacket::CodeOK);
			requestLogin(rxpacket);
		} else {
			requestDisconnect();
		}
	}
}

void ICEClientSession::processReplyConnection(ICEPacket* rxpacket)
{
	if ( rxpacket->getCode() == ICEPacket::CodeOK ) {
		setState(SessionConnected);
		requestLogin( rxpacket );
	} else {
		/** TODO - should emit this as a text message to UI*/
		fprintf( stderr, "reply connection code=%d\n", rxpacket->getCode() );
	}
}

void ICEClientSession::processReplyConnectNewLoginID(ICEPacket* rxpacket)
{
	if ( rxpacket->getCode() == ICEPacket::CodeOK ) {
		setState(SessionConnected);
		requestLogin( rxpacket );
	} else {
		/** TODO - should emit this as a text message to UI*/
		fprintf( stderr, "reply new login id code=%d\n", rxpacket->getCode() );
	}
}

void ICEClientSession::processReplyDeleteLogin ( ICEPacket* rxpacket )
{
	if ( rxpacket->getCode() == ICEPacket::CodeOK )
	{
		requestDisconnect();
	}
}

void ICEClientSession::processAsyncPopupMessage ( ICEPacket* rxpacket )
{
	if ( rxpacket->getCode() == ICEPacket::CodeOK )
	{
		QString text = rxpacket->getString();
		QMessageBox::information( NULL, "Async Popup Message", text );
	}
}

void ICEClientSession::asyncPopupMessage(QString text,ICEPacket::PacketCode code)
{
	QMessageBox::information( NULL, "Async Popup Message", text );
}

void ICEClientSession::rx ( ICEPacket rxpacket )
{
	QApplication::processEvents();
	switch ( rxpacket.getType() )
	{
		case ICEPacket::Empty:                       break;
		case ICEPacket::Ping:                        break;
		case ICEPacket::Fault:                       break;

		case ICEPacket::RequestConnection:           break;
		case ICEPacket::RequestDisconnect:           break;
		case ICEPacket::RequestConnectNewLoginID:    break;
		case ICEPacket::RequestLogin:                break;
		case ICEPacket::RequestDeleteLogin:          break;
		case ICEPacket::RequestSession:              break;
		case ICEPacket::RequestPasswordChange:       break;

		case ICEPacket::ReplyConnectNewLoginID:      processReplyConnectNewLoginID ( &rxpacket );break;
		case ICEPacket::ReplyConnection:             processReplyConnection ( &rxpacket );      break;
		case ICEPacket::ReplyDisconnect:             processReplyDisconnect ( &rxpacket );      break;
		case ICEPacket::ReplyLogin:                  processReplyLogin ( &rxpacket );           break;
		case ICEPacket::ReplyDeleteLogin:            processReplyDeleteLogin ( &rxpacket );     break;
		case ICEPacket::ReplySession:                processReplySession ( &rxpacket );         break;
		case ICEPacket::ReplyPasswordChange:         break;
		case ICEPacket::AsyncPopupMessage:           processAsyncPopupMessage ( &rxpacket );    break;
		default:
			inherited::rx( rxpacket );
			break;	
	}
	/*
	** Handle fault codes...
	*/
	if ( rxpacket.getCode() != ICEPacket::CodeOK )
	{
		QString aux;
		switch ( rxpacket.getType() )
		{
			case ICEPacket::ReplyConnectNewLoginID:
				aux = QString ( "ReplyConnectNewLoginID" ) + rxpacket.getLogin();
				break;
			case ICEPacket::ReplyConnection:
				aux = QString ( "ReplyConnection" ) + rxpacket.getLogin();
				break;
			case ICEPacket::ReplyLogin:
				aux = QString ( "ReplyLogin" ) + rxpacket.getLogin();
				break;
			case ICEPacket::ReplyDeleteLogin:
				aux = QString ( "ReplyDeleteLogin" ) + rxpacket.getLogin();
				break;
			case ICEPacket::ReplySession:
				aux = QString ( "ReplySession" ) + rxpacket.getLogin();
				break;
			case ICEPacket::ReplyPasswordChange:
				aux = QString ( "ReplyPasswordChange" ) + rxpacket.getLogin();
				break;
		}
		emit packetError ( rxpacket.getCode(),aux );
		if ( rxpacket.getCode() != ICEPacket::CodeOK )
		{
			fprintf ( stderr, "%s: code=%d\n", aux.toAscii().data(), ( int ) rxpacket.getCode() );
		}
	}
}


void ICEClientSession::newAccount()
{
	mRequestingNewID = true;
	if ( state() != SessionClosed )
	{
		close();
	}
	else
	{
		connectToHost();
	}
}

void ICEClientSession::deleteAccount()
{
	if ( state() == SessionValidated )
	{
		requestDeleteLogin();
	}
	else
	{
		mDeletingAccount=true;
		connectToHost();
	}
}

void ICEClientSession::changePassword()
{
	fprintf ( stderr, "FIXME - change password\n" );
}

void ICEClientSession::connectToHost()
{
	socket()->connectToHost ( mHostName,mPort );
}

void ICEClientSession::connectToHost ( QString host,int port )
{
	setHost ( host,port );
	connectToHost();
}

void ICEClientSession::disconnectFromHost()
{
	socket()->disconnectFromHost();
}

void ICEClientSession::setHost ( QString host,int port )
{
	mHostName = host;
	mPort = port;
	emit hostChanged ( host,port );
}

void ICEClientSession::timerEvent(QTimerEvent* e)
{
	inherited::timerEvent(e);
}


void ICEClientSession::close()
{
	inherited::close();
}

void ICEClientSession::slotConnected()
{
	fprintf( stderr, "client session connected\n" );
	if ( mRequestingNewID ) {
		requestConnectNewLoginID();
		mRequestingNewID = false;
	} else if ( mDeletingAccount ) {
		requestDeleteLogin();
		mDeletingAccount = false;
	} else {
		requestConnection();
	}
}

void ICEClientSession::slotDisconnected()
{
	fprintf( stderr, "client session disconnected\n" );
	setState(SessionClosed);
	quit();
	if (mRequestingNewID )
		connectToHost();

}

void ICEClientSession::slotError(QAbstractSocket::SocketError err)
{
	fprintf( stderr, "client session error %d\n", err );
}

void ICEClientSession::slotHostFound()
{
	fprintf( stderr, "client session host found\n" );
}

void ICEClientSession::slotStateChanged(QAbstractSocket::SocketState state)
{
	switch ( state ) {
		case QAbstractSocket::UnconnectedState:
			emit statusChanged(ICEStatus::ConnectNo);
			emit clear();
			break;
		case QAbstractSocket::HostLookupState:
		case QAbstractSocket::ConnectingState:
			emit statusChanged(ICEStatus::ConnectCreating);
			break;
		case QAbstractSocket::ConnectedState:
			emit statusChanged(ICEStatus::ConnectEstablished);
			break;
	}
}

void ICEClientSession::go( const QString& location ) 
{
	if ( state() == SessionValidated ) {
		ICESessionObject* so = construct(ICESessionObject::TypeUiTransport,NULL,&location.toAscii());
	} else {
		QUrl url( location );
		mInitialLocation = location;
		if ( url.host() != hostname() || url.port() != port() ) {
			// FIXME - fix the port here...
			setHost ( url.host(), 8088 /* url.port() */ );
			disconnectFromHost();
			connectToHost();
		}
	}
}
