/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   info@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the teice of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ICECLIENTSESSION_H
#define ICECLIENTSESSION_H

#include <QObject>
#include <QTcpSocket>
#include <QThread>
#include <QTimerEvent>
#include <QQueue>
#include <QMainWindow>
#include <QUrl>

#include "icesession.h"
#include "icestatus.h"

/**
	@author Michael Sharkey
 */
class ICEClientSession : public ICESession
{
		Q_OBJECT
	public:

		ICEClientSession ( QObject *parent = 0 );
		virtual ~ICEClientSession();

		void			setMainWindow(QMainWindow* window) {mMainWindow=window;}
		QMainWindow*	mainWindow() {return mMainWindow;}
		
		QString     hostname()  {return mHostName;}
		int         port()      {return mPort;}

		bool        copy ( QUrl fromUrl, QUrl toUrl );
	
		virtual		ICESessionObject* construct(ICESessionObject::ObjectType,QByteArray* id=NULL,QByteArray* data=NULL);

	signals:
		void		clear();
		void        popupMessage ( QString text );
		void        hostChanged ( QString host,int port );
		void		statusChanged ( ICEStatus::ConnectStatus );

	public slots:
		virtual void    	asyncPopupMessage(QString text,ICEPacket::PacketCode code=ICEPacket::CodeOK);
				void		go(const QString& location );
				void        setHost ( QString host,int port );
				void        connectToHost();
				void        connectToHost ( QString host,int port );
				void        disconnectFromHost();
				void        newAccount();
				void        deleteAccount();
				void        changePassword();

				void        rx ( ICEPacket rxpacket );           /* receive a packet */

	protected:
		
		void        requestConnection();
		void        requestConnectNewLoginID();
		void        requestDisconnect ( ICEPacket::PacketCode code=ICEPacket::CodeOK );
		void        requestLogin ( ICEPacket* );
		void        requestDeleteLogin();
		void        requestSession ( ICEPacket* );

		virtual		void		replySessionObjectConstruct(ICEPacket*,ICEPacket::PacketCode code=ICEPacket::CodeOK);

		
		void        processReplyDisconnect ( ICEPacket* );
		void        processReplyConnection ( ICEPacket* );
		void        processReplyConnectNewLoginID ( ICEPacket* );
		void        processReplyLogin ( ICEPacket* );
		void        processReplyDeleteLogin ( ICEPacket* );
		void        processReplySession ( ICEPacket* );
		void        processAsyncPopupMessage ( ICEPacket* );

	protected:

		void        timerEvent ( QTimerEvent* e );

	protected slots:
		void        close();
		void        slotConnected();
		void        slotDisconnected();
		void        slotError ( QAbstractSocket::SocketError err );
		void        slotHostFound();
		void        slotStateChanged ( QAbstractSocket::SocketState state );

	private:
		QByteArray  		mChallenge;                     /* login challenge data */
		QString     		mHostName;
		int         		mPort;
		bool        		mRequestingNewID;
		bool        		mDeletingAccount;
		QMainWindow* 		mMainWindow;
		QString				mInitialLocation;
};

#endif
