/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   info@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************t************************************/
#include "iceclientui.h"
#include "icesession.h"
#include "icesessionobject.h"
#include "iceuiloader.h"
#include "iceclientuielement.h"
#include "iceclientsessionobjectuielementtransport.h"

#include <QFileInfo>
#include <QFile>
#include <QEvent>

#define inherited ICEUi

ICEClientUi::ICEClientUi(ICEScriptThread *parent)
 : inherited(parent)
{
}


ICEClientUi::~ICEClientUi()
{
	destroyAll();
}

/**
** \brief destroy all class variable in preparation for a re-load.
**/
void ICEClientUi::destroyAll()
{
	if ( widget() != NULL )
	{
		delete widget(); /** delete the root widget (form) and all it's children */
		for ( int i=0; i < mElementTransportMap.count(); i++ )
		{
			ICEElementTransportMapItem* item = mElementTransportMap.takeLast();
			delete item;
		}
	}
}

void ICEClientUi::load( QString ui )
{
	inherited::load( ui );
}

/**
** \brief load a ui bytestream which is encoded in .ui XML format.
**/
void ICEClientUi::load( QByteArray ui )
{
	if ( !ui.isEmpty() ) {
		destroyAll();
		ICEElementTransportMapItem* item  = new ICEElementTransportMapItem( this );
		item->setObject( ICEUiLoader().load(ui) );
		if ( item->object() != NULL ) {
			QList<QWidget*> mWidgets;
			mElementTransportMap.append(item); /** root object (form) */
			mWidgets = item->object()->findChildren<QWidget *>();
			for(int i=0; i < mWidgets.count(); i++)
			{
				ICEClientSessionObjectUiElementTransport* uiElementTransport = (ICEClientSessionObjectUiElementTransport*)(session()->construct( ICESessionObject::TypeUiElementTransport ));
				if ( uiElementTransport != NULL )
				{
					item = new ICEElementTransportMapItem( this );
					item->setObject( mWidgets[i] );	
					uiElementTransport->obtain();
					item->setSessionObjectId( uiElementTransport->id() );
					uiElementTransport->release();
					mElementTransportMap.append(item);
					mWidgets[i]->installEventFilter(this);
				}
			}
		} else {
			emit loadFailed( tr( "failed to load UI" ) );
		}
	}
}

QWidget* ICEClientUi::widget()
{
	if ( mElementTransportMap.count() > 0 )
	{
		QWidget* w = ((QWidget*)mElementTransportMap[0]->object());
		return w;
	}
	return NULL;
}

QByteArray ICEClientUi::findSessionObjectId( QObject* o )
{
	for( int i=0; i < mElementTransportMap.count(); i++ )
	{
		ICEElementTransportMapItem* item = mElementTransportMap[i];
		if ( item->object() == o )
		{
			return item->sessionObjectId();
		}
	}
	return QByteArray();
}

bool ICEClientUi::eventFilter(QObject *watched, QEvent *e)
{
	QByteArray id =  findSessionObjectId( watched );
	if ( !id.isEmpty() )
	{
		ICESessionObject* so = ICESessionObject::obtain( id );
		if ( so != NULL )
		{
			ICEEvent event((qint32)e->type());
			so->session()->requestSessionObjectEvent( so, event );
			so->release();
		}
	}
	return inherited::eventFilter( watched, e );
}

