/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   mike@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the teice of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ICECRYPTLOKI91_H
#define ICECRYPTLOKI91_H

#include <icecrypt.h>

/**
	@author R. Michael Sharkey <mike@pikeaero.com>
*/

#include <QObject>
#include <QByteArray>
#include <QString>

/**
* LOKI91
*
*/

#define LOKIBLK 8               /** No of bytes in a LOKI data-block          */
#define ROUNDS  16              /** No of LOKI rounds                         */

class ICECryptLOKI91 : public ICECrypt
{
		Q_OBJECT

	public:
		ICECryptLOKI91();
		~ICECryptLOKI91();

		virtual void        cipher ( bool encrypt, QString key, QByteArray* in, QByteArray* out );

	private:
		void 		init ( unsigned char* key );
		void		encipher ( unsigned char* b );
		void		decipher ( unsigned char *b );
		void		bswap ( unsigned char *cb );
		quint32		f ( quint32 r, quint32 k );
		qint16		s ( quint32 i );
		void		perm32 ( quint32 *out, quint32 *in, char *perm );
		qint16		mult8 ( qint16 a, qint16 b, qint16 gen );
		qint16		exp8 ( qint16 base, qint16 exponent, qint16 gen );

		quint32		loki_subkeys[ROUNDS];		/** subkeys at the 16 rounds */
		quint32		lokikey[2];     			/** 64-bit key used by LOKI routines */
		quint32 	last_xdatal;
		quint32 	last_xdatar;
		bool 		cbc;

};

#endif
