/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   mike@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the teice of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "icecryptseal.h"

extern void dump ( QByteArray* bytes );


#define ALG_OK 0

ICECryptSEAL::ICECryptSEAL()
: ICECrypt()
{}


ICECryptSEAL::~ICECryptSEAL()
{}


void ICECryptSEAL::g ( unsigned char* in, unsigned int i, quint32* h )
{
	quint32 h0;
	quint32 h1;
	quint32 h2;
	quint32 h3;
	quint32 h4;
	quint32 a;
	quint32 b;
	quint32 c;
	quint32 d;
	quint32 e;
	unsigned char *kp;
	quint32 w[80];
	quint32 temp;

	kp = in;
	h0 = WORD( kp ); kp += 4;
	h1 = WORD( kp ); kp += 4;
	h2 = WORD( kp ); kp += 4;
	h3 = WORD( kp ); kp += 4;
	h4 = WORD( kp ); kp += 4;

	w[0] = i;
	for ( i=1;i<16;i++ )
		w[i] = 0;
	for ( i=16;i<80;i++ )
		w[i] = w[i-3]^w[i-8]^w[i-14]^w[i-16];

	a = h0;
	b = h1;
	c = h2;
	d = h3;
	e = h4;

	for ( i=0;i<20;i++ )
	{
		temp = ROT27 ( a ) + F1 ( b, c, d ) + e + w[i] + 0x5a827999;
		e = d;
		d = c;
		c = ROT2 ( b );
		b = a;
		a = temp;
	}
	for ( i=20;i<40;i++ )
	{
		temp = ROT27 ( a ) + F2 ( b, c, d ) + e + w[i] + 0x6ed9eba1;
		e = d;
		d = c;
		c = ROT2 ( b );
		b = a;
		a = temp;
	}
	for ( i=40;i<60;i++ )
	{
		temp = ROT27 ( a ) + F3 ( b, c, d ) + e + w[i] + 0x8f1bbcdc;
		e = d;
		d = c;
		c = ROT2 ( b );
		b = a;
		a = temp;
	}
	for ( i=60;i<80;i++ )
	{
		temp = ROT27 ( a ) + F4 ( b, c, d ) + e + w[i] + 0xca62c1d6;
		e = d;
		d = c;
		c = ROT2 ( b );
		b = a;
		a = temp;
	}
	h[0] = h0+a;
	h[1] = h1+b;
	h[2] = h2+c;
	h[3] = h3+d;
	h[4] = h4+e;
}

quint32 ICECryptSEAL::gamma ( unsigned char* a, int i )
{
	quint32 h[5];
	( void ) g ( a, i/5, h );
	return h[i % 5];
}

void ICECryptSEAL::init ( unsigned char *key )
{
	int i;
	quint32 h[5];

	for ( i=0;i<510;i+=5 )
		g ( key, i/5, & ( t[i] ) );
	/* horrible special case for the end */
	g ( key, 510/5, h );
	for ( i=510;i<512;i++ )
		t[i] = h[i-510];
	/* 0x1000 mod 5 is +1, so have horrible special case for the start */
	g ( key, ( -1+0x1000 ) /5, h );
	for ( i=0;i<4;i++ )
		s[i] = h[i+1];
	for ( i=4;i<254;i+=5 )
		g ( key, ( i+0x1000 ) /5, & ( s[i] ) );
	/* horrible special case for the end */
	g ( key, ( 254+0x1000 ) /5, h );
	for ( i=254;i<256;i++ )
		s[i] = h[i-254];
	/* 0x2000 mod 5 is +2, so have horrible special case at the start */
	g ( key, ( -2+0x2000 ) /5, h );
	for ( i=0;i<3;i++ )
		r[i] = h[i+2];
	for ( i=3;i<13;i+=5 )
		g ( key, ( i+0x2000 ) /5, & ( r[i] ) );
	/* horrible special case for the end */
	g ( key, ( 13+0x2000 ) /5, h );
	for ( i=13;i<16;i++ )
		r[i] = h[i-13];
}

void ICECryptSEAL::seal ( quint32 in, quint32 *out )
{
	int i;
	int j;
	int l;
	quint32 a;
	quint32 b;
	quint32 c;
	quint32 d;
	quint16 p;
	quint16 q;
	quint32 n1;
	quint32 n2;
	quint32 n3;
	quint32 n4;
	quint32 *wp;

	wp = out;

	for ( l=0;l<4;l++ )
	{
		a = in ^ r[4*l];
		b = ROT8 ( in ) ^ r[4*l+1];
		c = ROT16 ( in ) ^ r[4*l+2];
		d = ROT24 ( in ) ^ r[4*l+3];

		for ( j=0;j<2;j++ )
		{
			p = a & 0x7fc;
			b += t[p/4];
			a = ROT9 ( a );

			p = b & 0x7fc;
			c += t[p/4];
			b = ROT9 ( b );

			p = c & 0x7fc;
			d += t[p/4];
			c = ROT9 ( c );

			p = d & 0x7fc;
			a += t[p/4];
			d = ROT9 ( d );

		}
		n1 = d;
		n2 = b;
		n3 = a;
		n4 = c;

		p = a & 0x7fc;
		b += t[p/4];
		a = ROT9 ( a );

		p = b & 0x7fc;
		c += t[p/4];
		b = ROT9 ( b );

		p = c & 0x7fc;
		d += t[p/4];
		c = ROT9 ( c );

		p = d & 0x7fc;
		a += t[p/4];
		d = ROT9 ( d );

		/* This generates 64 32-bit words, or 256 bytes of keystream. */
		for ( i=0;i<64;i++ )
		{
			p = a & 0x7fc;
			b += t[p/4];
			a = ROT9 ( a );
			b ^= a;

			q = b & 0x7fc;
			c ^= t[q/4];
			b = ROT9 ( b );
			c += b;

			p = ( p+c ) & 0x7fc;
			d += t[p/4];
			c = ROT9 ( c );
			d ^= c;

			q = ( q+d ) & 0x7fc;
			a ^= t[q/4];
			d = ROT9 ( d );
			a += d;

			p = ( p+a ) & 0x7fc;
			b ^= t[p/4];
			a = ROT9 ( a );

			q = ( q+b ) & 0x7fc;
			c += t[q/4];
			b = ROT9 ( b );

			p = ( p+c ) & 0x7fc;
			d ^= t[p/4];
			c = ROT9 ( c );

			q = ( q+d ) & 0x7fc;
			a += t[q/4];
			d = ROT9 ( d );

			*wp = b + s[4*i];
			wp++;
			*wp = c ^ s[4*i+1];
			wp++;
			*wp = d + s[4*i+2];
			wp++;
			*wp = a ^ s[4*i+3];
			wp++;

			if ( i & 1 )
			{
				a += n3;
				c += n4;
			}
			else
			{
				a += n1;
				c += n2;
			}

		}
	}
}

/* Added call to refill ks_buf and reset counter and ks_pos. */
void ICECryptSEAL::refill_buffer()
{
	seal ( counter,ks_buf );
	counter++;
	ks_pos = 0;
}

void ICECryptSEAL::initkey ( unsigned char* key )
{
	init ( key );
	counter = 0;  /* By default, init to zero. */
	ks_pos = WORDS_PER_SEAL_CALL;
	/* Refill keystream buffer on next call. */
}

/* This encrypts the next w words with SEAL. */
void ICECryptSEAL::encipher ( quint32 *data_ptr, int w )
{
	int i;

	for ( i=0;i<w;i++ )
	{
		if ( ks_pos>=WORDS_PER_SEAL_CALL ) refill_buffer();
		data_ptr[i]^=ks_buf[ks_pos];
		ks_pos++;
	}
}

void ICECryptSEAL::decipher ( quint32 *data_ptr, int w )
{
	encipher ( data_ptr,w );
}

void ICECryptSEAL::resynch ( quint32 synch_word )
{
	counter = synch_word;
	ks_pos = WORDS_PER_SEAL_CALL;
}

void ICECryptSEAL::cipher ( bool encrypt, QString key, QByteArray* in, QByteArray* out )
{
	if ( !mInitialized )
	{
		/* FIXME translate ASCII key into 160 bit key. */
		fprintf ( stderr, "FIXME - SEAL - translate ASCII key into 160 bit key.\n" );
		QByteArray xkey = key.toLatin1(); /* FIXME - key size hack.... */
		initkey ( ( unsigned char* ) xkey.data() );
		mInitialized = true;
	}
	out->resize ( in->size() );
	for ( int index = 0; index < in->size(); index+=4 )
	{
		quint32 data=0;
		quint32 remaining=in->size()-index;
		if ( remaining < 4 )
		{
			int pad = ( 4-remaining );
			in->resize ( in->size() + pad );
			out->resize ( out->size() + pad );
		}
		data |= ( ( ( ( unsigned char ) in->at ( index+0 ) ) << 24 ) & 0xFF000000 );
		data |= ( ( ( ( unsigned char ) in->at ( index+1 ) ) << 16 ) & 0x00FF0000 );
		data |= ( ( ( ( unsigned char ) in->at ( index+2 ) ) << 8 ) & 0x0000FF00 );
		data |= ( ( ( unsigned char ) in->at ( index+3 ) ) & 0x000000FF );
		if ( encrypt )
		{
			encipher ( &data, 1 );
		}
		else
		{
			decipher ( &data, 1 );
		}
		out->data() [index+0] = ( ( data >> 24 ) & 0xFF );
		out->data() [index+1] = ( ( data >> 16 ) & 0xFF );
		out->data() [index+2] = ( ( data >> 8 ) & 0xFF );
		out->data() [index+3] = ( data & 0xFF );
	}
}


