/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   mike@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the teice of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ICESEAL_H
#define ICESEAL_H

#include <icecrypt.h>

/**
	@author R. Michael Sharkey <mike@pikeaero.com>
*/

#define WORDS_PER_SEAL_CALL 1024

class ICECryptSEAL : public ICECrypt
{
	Q_OBJECT
	public:
		ICECryptSEAL();
		~ICECryptSEAL();
		virtual void        cipher( bool encrypt, QString key, QByteArray* in, QByteArray* out );
    
	private:

		inline quint32 ROT2(quint32 x)	{ return (((x) >> 2)	| ((x) << 30)); }
		inline quint32 ROT9(quint32 x)	{ return (((x) >> 9)	| ((x) << 23)); }
		inline quint32 ROT8(quint32 x)	{ return (((x) >> 8)	| ((x) << 24)); }
		inline quint32 ROT16(quint32 x)	{ return (((x) >> 16)	| ((x) << 16)); }
		inline quint32 ROT24(quint32 x)	{ return (((x) >> 24)	| ((x) << 8));	}
		inline quint32 ROT27(quint32 x)	{ return (((x) >> 27)	| ((x) << 5));	}

		inline quint32 WORD(unsigned char* cp)	{ return ((cp[0] << 24)|(cp[1] << 16)|(cp[2] << 8)|(cp[3])); }

		inline quint32 F1(quint32 x, quint32 y, quint32 z) { return (((x) & (y)) | ((~(x)) & (z))); }
		inline quint32 F2(quint32 x, quint32 y, quint32 z) { return ((x)^(y)^(z)); }
		inline quint32 F3(quint32 x, quint32 y, quint32 z) { return (((x) & (y)) | ((x) & (z)) | ((y) & (z))); }
		inline quint32 F4(quint32 x, quint32 y, quint32 z) { return ((x)^(y)^(z)); }

		void g(unsigned char* in, unsigned int i, quint32* h);
		quint32 gamma(unsigned char* a, int i);
		void seal(quint32 in, quint32 *out);
		void refill_buffer();
		void init(unsigned char *key);
		void encipher(quint32 *data_ptr, int w);
		void decipher(quint32 *data_ptr, int w);
		void resynch(quint32 synch_word);
		void initkey(unsigned char *key);
    
		quint32 t[520]; /** 512 rounded up to a multiple of 5 + 5*/
		quint32 s[265]; /** 256 rounded up to a multiple of 5 + 5*/
		quint32 r[20];  /** 16 rounded up to multiple of 5 */
		quint32 counter; /** 32-bit synch value. */
		quint32 ks_buf[WORDS_PER_SEAL_CALL];
		int ks_pos;
};

#endif
