/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   info@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ICESQL_H
#define ICESQL_H

#include "icesession.h"

#include <QObject>
#include <QFile>
#include <QList>
#include <QtSql>

/**
Wrapper class for SQL access. Loads a list of SQL queries from /opt/ice/sql and
then provides a var-arg method for executing the queries.

	@author R. Michael Sharkey <mike@pikeaero.com>
*/


class ICESqlItem : public QObject
{
	Q_OBJECT
	public:
		ICESqlItem(QString& type, QString& name, QString& text, QObject *parent = 0)
		: QObject( parent )
		, mType( type )
		, mName( name )
		, mText( text )
		{
		}
		~ICESqlItem() {}
		
		inline QString& type() {return mType;}
		inline QString& name() {return mName;}
		inline QString& text() {return mText;}
		
		inline void setType( QString type ) { mType = type; }
		inline void setName( QString name ) { mName = name; }
		inline void setText( QString text ) { mText = text; }
		
	private:
		QString	mType;		/** type = "create", "drop", "user", etc... */
		QString	mName;		/** name = "ice_config", "ice_content", etc... */
		QString	mText;		/** The text of the query with printf style substitution characters embedded */
};

class ICESql : public QObject
{
	Q_OBJECT
	public:
		ICESql(QObject *parent = 0);
		~ICESql();
	
		/**
		 ** Find a query of type, and name, and pass a variable
		 ** list of arguments...
		 */
		static QSqlQuery query( QString type, QString name, ... );
		static QSqlQuery queryAll( QString type ); /** Execute all queries of a particular type */
		static QString find( QString type, QString name );
		bool load( QString path );
	signals:
		void		statusMessage(void*,ICESession::MessageReason reason,QString message);
	private:
		static QList<ICESqlItem*>	mItemList;
		static ICESql* mInstance;
};

#endif
