/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   info@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "iceevent.h"

#include <QBuffer>
#include <QXmlInputSource>
#include <QXmlSimpleReader>
#include <QByteArray>

#define inherited QXmlContentHandler

QDataStream & operator>>(QDataStream &in, const ICEEvent &fi)
{
	ICEEvent* me = (ICEEvent*)&fi;
	QString xml;
	in >> xml; me->fromXml(xml);
	return in;
}

QDataStream & operator<<(QDataStream &out, const ICEEvent &fi)
{
	ICEEvent* me = (ICEEvent*)&fi;
	out << me->toXml();
	return out;
}

QDataStream & operator>>(QDataStream &in, ICEEvent &fi)
{
	ICEEvent* me = (ICEEvent*)&fi;
	QString xml;
	in >> xml; me->fromXml(xml);
	return in;
}

QDataStream & operator<<(QDataStream &out, ICEEvent &fi)
{
	ICEEvent* me = (ICEEvent*)&fi;
	out << me->toXml();
	return out;
}

ICEEvent::ICEEvent(qint32 eventType)
: mType(eventType)
, mTypeElement(0)
{
}


ICEEvent::~ICEEvent()
{
}

/**
	\brief Deep Copy from other object.

	\param other Other event to copy from.
 **/
ICEEvent& ICEEvent::operator=(ICEEvent& other)
{
	copy(other);
	return *this;
}

void ICEEvent::copy(ICEEvent& other)
{
	setType( other.type() );
}

QString ICEEvent::toXml()
{
	return "<type>" + QString::number(type()) + "</type>";
}

void ICEEvent::fromXml(QString xml)
{
	QBuffer buffer;
	QXmlSimpleReader reader;
	buffer.setData( xml.toAscii() );
	QXmlInputSource source( &buffer );
	reader.setContentHandler( this );
	reader.parse( &source );
}

bool ICEEvent::characters ( const QString & ch )
{
	bool rc=true;
	if ( mTypeElement )
	{
		mType = ch.toInt();
	}
	return rc;
}

bool ICEEvent::endDocument ()
{
	bool rc=true;
	return rc;
}

bool ICEEvent::endElement ( const QString & namespaceURI, const QString & localName, const QString & qName )
{
	bool rc=true;
	if ( localName.toLower() == "type" )
	{
		--mTypeElement;
	}
	return rc;
}

bool ICEEvent::endPrefixMapping ( const QString & prefix )
{
	bool rc=true;
	return rc;
}

QString ICEEvent::errorString () const
{
	QString msg="XML parsing error";
	return msg;
}

bool ICEEvent::ignorableWhitespace ( const QString & ch )
{
	bool rc=true;
	return rc;
}

bool ICEEvent::processingInstruction ( const QString & target, const QString & data )
{
	bool rc=true;
	return rc;
}

void ICEEvent::setDocumentLocator ( QXmlLocator * locator )
{
}

bool ICEEvent::skippedEntity ( const QString & name )
{
	bool rc=true;
	return rc;
}

bool ICEEvent::startDocument ()
{
	bool rc=true;
	mTypeElement=0;
	return rc;
}

bool ICEEvent::startElement ( const QString & namespaceURI, const QString & localName, const QString & qName, const QXmlAttributes & atts )
{
	bool rc=true;
	if ( localName.toLower() == "type" )
	{
		++mTypeElement;
	}
	else
	{
		rc=false;
	}
	return rc;
}

bool ICEEvent::startPrefixMapping ( const QString & prefix, const QString & uri )
{
	bool rc=true;
	return rc;
}
