/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   info@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ICEEVENT_H
#define ICEEVENT_H

#include <QEvent>
#include <QXmlContentHandler>

/**
Serializable event wrapper transport

	@author R. Michael Sharkey <mike@pikeaero.com>
*/
class ICEEvent : QXmlContentHandler
{
	public:
		ICEEvent(qint32 eventType=0);
		~ICEEvent();
		ICEEvent		&operator=(ICEEvent &other);
		inline	qint32	type() {return mType;}
		virtual void	setType(int eventType) {mType=eventType;}
		virtual	QString	toXml();
		virtual	void	fromXml(QString xml);

		/** Xml content handler overrides */
		virtual bool	characters ( const QString & ch );
		virtual bool	endDocument ();
		virtual bool	endElement ( const QString & namespaceURI, const QString & localName, const QString & qName );
		virtual bool	endPrefixMapping ( const QString & prefix );
		virtual QString errorString () const;
		virtual bool	ignorableWhitespace ( const QString & ch );
		virtual bool	processingInstruction ( const QString & target, const QString & data );
		virtual void	setDocumentLocator ( QXmlLocator * locator );
		virtual bool	skippedEntity ( const QString & name );
		virtual bool	startDocument ();
		virtual bool	startElement ( const QString & namespaceURI, const QString & localName, const QString & qName, const QXmlAttributes & atts );
		virtual bool	startPrefixMapping ( const QString & prefix, const QString & uri ); 
	protected:
		virtual	void	copy(ICEEvent& other);
	private:
		qint32	mType;
		int		mTypeElement;	/** <type> element depth */
};

QDataStream & operator<<( QDataStream& id, const ICEEvent& i );
QDataStream & operator>>( QDataStream& id, const ICEEvent& i );
QDataStream & operator<<( QDataStream& id,  ICEEvent& i );
QDataStream & operator>>( QDataStream& id,  ICEEvent& i );


#endif
