/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   info@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "icenetwork.h"

ICENetwork::ICENetwork(QObject *parent,bool server)
: QObject(parent)
, mNetworkError((NetworkError)0)
, mNetworkState((NetworkState)0)
, mProtocol(NULL)
{
	mProtocol = new ICEProtocol(parent,&mSocket,server);
	QObject::connect(&mSocket,SIGNAL(connected()),this,SLOT(slotConnected()));
	QObject::connect(&mSocket,SIGNAL(disconnected()),this,SLOT(slotDisconnected()));
	QObject::connect(&mSocket,SIGNAL(error(QAbstractSocket::SocketError)),
					  this,SLOT(slotError(QAbstractSocket::SocketError)));
	QObject::connect(&mSocket,SIGNAL(hostFound()),this,SLOT(slotHostFound()));
	QObject::connect(&mSocket,SIGNAL(stateChanged(QAbstractSocket::SocketState)),
					  this, SLOT(slotStateChanged(QAbstractSocket::SocketState)));
}


ICENetwork::~ICENetwork()
{
}

void ICENetwork::disconnectFromHost()
{
	mSocket.close();
}

void ICENetwork::connectToHost(QString host,int port)
{
	mSocket.connectToHost(host,port);
}

void ICENetwork::login(QString userid,QString password)
{
}

void ICENetwork::slotConnected()
{
	emit connected();
}

void ICENetwork::slotDisconnected()
{
	emit disconnected();
}

void ICENetwork::slotError(QAbstractSocket::SocketError err)
{
	mNetworkError = (NetworkError)err;
	emit error(mNetworkError);
}

void ICENetwork::slotHostFound()
{
	emit hostFound();
}

void ICENetwork::slotStateChanged(QAbstractSocket::SocketState state)
{
	mNetworkState = (NetworkState)state;
	emit stateChanged(mNetworkState);
}



