/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   info@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ICENETWORK_H
#define ICENETWORK_H

#include <QObject>

#include "icesocket.h"
#include "iceprotocol.h"

/**
	@author R. Michael Sharkey <mike@pikeaero.com>
*/
class ICENetwork : public QObject
{
	Q_OBJECT
	public:
		ICENetwork(QObject *parent = 0,bool server=false);
		~ICENetwork();

		typedef enum {
			/** this section cloned from QAbstractSocket::SocketError */
			UnknownSocketError=-1,
			ConnectionRefusedError=0,
			RemoteHostClosed,
			HostNotFound,
			SocketAccessError,
			SocketResourceError,
			SocketTimeoutError,
			DatagramTooLargeError,
			GeneralNetworkError,
			AddressInUseError,
			SocketAddressNotAvailableError,
			UnsupportedSocketOperationError,
		} NetworkError;
		
		typedef enum {
			/** this section cloned from QAbstractSocket::SocketError */
			UnconnectedState=0,
			HostLookupState,
			ConnectingState,
			ConnectedState,
			BoundState,
			ListeningState,
			ClosingState,
		} NetworkState;
	
		ICEProtocol* protocol() {return mProtocol;}
	
		void        connectToHost(QString host,int port);
		void        disconnectFromHost();
		void        login(QString userid,QString passwd);
	
	signals:
		void        connected();
		void        disconnected();
		void        error(NetworkError networkError);
		void        hostFound();
		void        stateChanged(NetworkState state);
	
	protected slots:
		void        slotConnected();
		void        slotDisconnected();
		void        slotError(QAbstractSocket::SocketError err);
		void        slotHostFound();
		void        slotStateChanged(QAbstractSocket::SocketState state);
	
	private:
		NetworkError    mNetworkError;
		NetworkState    mNetworkState;
		ICESocket        mSocket;
		ICEProtocol*     mProtocol;
};

#endif
