/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   info@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <errno.h>
#include "iceprotocol.h"

ICEProtocol::ICEProtocol(QObject *parent,QTcpSocket* socket,bool server)
: QObject(parent)
, mServer(server)
, mSocket(socket)
{
	if ( socket != NULL ) {
		QObject::connect(&mPacketRX,SIGNAL(decoded(SSPacket)),this,SLOT(decoded(SSPacket))/*,Qt::DirectConnection*/);
		QObject::connect(&mPacketTX,SIGNAL(encoded(QByteArray)),this,SLOT(encoded(QByteArray)));
		QObject::connect(mSocket,SIGNAL(readyRead()),this,SLOT(rx()));
	} else {
		fprintf( stderr, "Protocol: Fatal NULL socket!\n" );
	}
	QObject::connect(&mCryptRX,SIGNAL(cryptError(int)),this,SLOT(cryptError(int)));
	QObject::connect(&mCryptTX,SIGNAL(cryptError(int)),this,SLOT(cryptError(int)));
}

ICEProtocol::~ICEProtocol()
{
}

void ICEProtocol::encoded(QByteArray bytes)
{
	/* a packet has been encoded, transmit it */
	qint64 rc;
	while ( (rc = socket()->write(bytes)) != bytes.size() ) {
		if ( rc < 0 ) {
			perror("socket write");
			break;
		}
		bytes.remove(0,rc); /* re3move what we've sent already */
	}
}

void ICEProtocol::decoded(ICEPacket packet)
{
	/* a packet has been decoded, handle it. */
	emit rx(packet);
}

void ICEProtocol::rx()
{
	mPacketRX.decode(socket(),&mCryptRX);
}

void ICEProtocol::tx(ICEPacket packet)
{
	mPacketTX = packet;
	mPacketTX.encode(&mCryptTX);
}

void ICEProtocol::cryptError(int err)
{
	fprintf( stderr, "crypt error %d\n", err );
}
