/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   info@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ICEPROTOCOL_H
#define ICEPROTOCOL_H

#include <QObject>
#include <QByteArray>
#include <QTcpSocket>
#include <QDataStream>

#include "icecrypt.h"
#include "icepacket.h"

/**
	@author Michael Sharkey
*/

class ICEProtocol : public QObject
{
	Q_OBJECT
	public:
		ICEProtocol(QObject *parent, QTcpSocket* socket, bool server=false);
		~ICEProtocol();
        
		QTcpSocket* socket() {return mSocket;}
		void        setRXKey(QString key) {mPacketRX.setKey(key);}
        
	signals:
		void        rx(ICEPacket packet);           /* signal received packet */
        
	public slots:
		void        rx();                           /* receive pending data */
		void        tx(ICEPacket packet);           /* transmit a packet */
        
		void        encoded(QByteArray bytes);      /* a packet has been encoded */
		void        decoded(ICEPacket packet);      /* a packet has been decoded */

	protected slots:
		void        cryptError(int);
        
	private:
		bool            mServer;        /* am I a server or client? */
		QTcpSocket*     mSocket;
		ICEPacket       mPacketTX;
		ICEPacket       mPacketRX;
		ICECrypt        mCryptTX;
		ICECrypt        mCryptRX;
};

#endif
