/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   info@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "icescriptlib.h"

#include <QDateTime>
#include <QEventLoop>

/**
 ** Script interface methods...
 */

ICEScriptThread* scriptThread()
{
	ICEScriptThread* script = (ICEScriptThread*)QThread::currentThread();
	return script;
}

QScriptValue scriptLib_ice_so_id(QScriptContext *context, QScriptEngine *engine)
{
	QScriptValue sessionObjectID = engine->globalObject().property( "ICE_SESSION_OBJECT_ID" );
	return 	sessionObjectID;
}

QScriptValue scriptLib_alert(QScriptContext *context, QScriptEngine *engine)
{
	if (context->argumentCount() >= 1)
	{
		QString msg = context->argument(0).toString();
		ICEScriptThread* script = scriptThread();
		script->alert( msg );
		return QScriptValue( engine, msg );
	}
	return QScriptValue( engine, "" );
}

QScriptValue scriptLib_sleep(QScriptContext *context, QScriptEngine *engine)
{
	if (context->argumentCount() >= 1)
	{
		int delay = context->argument(0).toInteger();
		ICEScriptThread* script = scriptThread();
		script->sleep( delay );
		return QScriptValue( engine, delay );
	}
	return QScriptValue( engine, 0 );
}


QScriptValue scriptLib_ice_exec(QScriptContext *context, QScriptEngine *engine)
{
	if (context->argumentCount() == 0)
	{
		ICEScriptThread* script = scriptThread();
		return QScriptValue( engine, script->public_exec() );
	}
	return QScriptValue(engine,-1);
}

QScriptValue scriptLib_ice_date_time(QScriptContext *context, QScriptEngine *engine)
{
	return QScriptValue ( engine, QDateTime::currentDateTime().toString() );
}

QScriptValue scriptLib_ice_process_events(QScriptContext *context, QScriptEngine *engine)
{
	QEventLoop loop;
	return QScriptValue ( engine, loop.processEvents() );
}

