/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   info@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "icescriptthread.h"
#include "icesessionobject.h"
#include "icesession.h"

#include "icescriptlib.h"

#include <QObject>
#include <QEventLoop>
#include <QScriptEngine>
#include <QScriptContext>
#include <QScriptValue>

ICEScriptThread::ICEScriptThread(ICESessionObject *parent)
 : QThread(NULL)
 , mSessionObject(parent)
 , mUi(NULL)
 , mScriptEngine(NULL)
{
	QObject::connect(this,SIGNAL(result(ICESessionObject*,QString)),parent,SIGNAL(statusMessage(ICESessionObject*,QString)),Qt::QueuedConnection);
	QObject::connect(this,SIGNAL(started()),this,SLOT(threadStarted()),Qt::QueuedConnection);
	QObject::connect(this,SIGNAL(finished()),this,SLOT(threadFinished()),Qt::QueuedConnection);
	QObject::connect(this,SIGNAL(terminated()),this,SLOT(threadTerminated()),Qt::QueuedConnection);
	QObject::connect(this,SIGNAL(issueAlert(QString)),parent,SLOT(alert(QString)),Qt::QueuedConnection);
}


ICEScriptThread::~ICEScriptThread()
{
	if ( mScriptEngine != NULL )
	{
		delete mScriptEngine;
		mScriptEngine=NULL;
	}
	if ( mUi != NULL )
	{
		delete mUi;
		mUi=NULL;
	}
}


ICESession* ICEScriptThread::session()
{
	return sessionObject()->session();
}
		
bool ICEScriptThread::load( QByteArray script )
{
	mScript = script;
	start();
}

/**
** - Perform script environment setup.
** - Setup timer event to trigger real startup.
** - Enter event loop -- timer event will start things rolling.
** We need the event loop running before we start running the script so that all of our
** cross-thread connects and socket notifiers will work.
*/

void ICEScriptThread::run()
{
	engine()->globalObject().setProperty("ICE_SESSION_OBJECT_ID", QScriptValue( engine(), QString( sessionObject()->id() ) ) );
	engine()->globalObject().setProperty("ice_so_id", engine()->newFunction(scriptLib_ice_so_id) );
	engine()->globalObject().setProperty("ice_date_time", engine()->newFunction(scriptLib_ice_date_time) );
	engine()->globalObject().setProperty("ice_process_events", engine()->newFunction(scriptLib_ice_process_events) );
	engine()->globalObject().setProperty("ice_exec", engine()->newFunction(scriptLib_ice_exec) );
	engine()->globalObject().setProperty("alert", engine()->newFunction(scriptLib_alert) );
	engine()->globalObject().setProperty("sleep", engine()->newFunction(scriptLib_sleep) );
	
	engine()->setProcessEventsInterval(20);
	mResult = engine()->evaluate( mScript ).toString();
}

void ICEScriptThread::setUi( ICEUi* ui )
{
	if ( mUi != NULL )
	{
		delete mUi;
	}
	mUi=ui;
}

void ICEScriptThread::threadStarted()
{
}

void ICEScriptThread::threadFinished()
{
	emit result( sessionObject(), mResult );
}

void ICEScriptThread::threadTerminated()
{
}

