/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   info@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ICESCRIPTTHREAD_H
#define ICESCRIPTTHREAD_H

#include <QThread>
#include <QByteArray>
#include <QScriptEngine>

#include "iceui.h"

class ICESessionObject;

/**
Script thread

	@author R. Michael Sharkey <mike@pikeaero.com>
*/
class ICEScriptThread : public QThread
{
	Q_OBJECT
	public:
		ICEScriptThread(ICESessionObject *parent);
		~ICEScriptThread();
		void 				run();
		QScriptEngine*		engine() 						{return mScriptEngine;}
		ICESessionObject*	sessionObject()					{return mSessionObject;}
		ICESession*			session();
	public slots:
		/**
			** A public interface to the exec9) method so that exec can be called from
			** a script belonging to this thread.
		*/
		virtual	int			public_exec()					{return exec();}
		virtual	bool 		load( QByteArray script );
		virtual	void 		sleep( int ms )					{msleep(ms);}
		virtual	void		setUi( ICEUi* ui );
		virtual	ICEUi*		ui()							{return mUi;}
		virtual void		alert( QString msg ) 			{ emit issueAlert( msg );}
	signals:
		void				issueAlert( QString msg );
		void 				result( ICESessionObject*, QString );
	protected:
		ICESessionObject*	mSessionObject;
		ICEUi*				mUi;
		QScriptEngine*		mScriptEngine;	/** The script engine */
	protected slots:
		virtual	void		threadStarted();
		virtual	void		threadFinished();
		virtual	void		threadTerminated();
	private:
		QByteArray 		mScript;
		QString 		mResult;
};

#endif
