/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   info@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "iceserversettings.h"

ICEServerSettings::ICEServerSettings(QObject *parent)
 : QObject(parent)
, mPort ( 8088 )
, mBacklog ( 40 )
, mLogging ( 9 )
, mSqlInitialize(false)
{
	restore();
}


ICEServerSettings::~ICEServerSettings()
{
}

void ICEServerSettings::save()
{
	QSettings settings(QSettings::NativeFormat, QSettings::UserScope, "pikeaero.com", "ICE Server");
	
	settings.beginGroup("sql");
		settings.setValue("sql_type",		mSqlType);		/** Qt database type (e.g. "QMYSQL"). */
		settings.setValue("sql_server",		mSqlServer);	/** SQL Database server address */
		settings.setValue("sql_database",	mSqlDatabase);	/** SQL Database name */
		settings.setValue("sql_user",		mSqlUser);		/** SQL Database user name (login) */
		settings.setValue("sql_password",	mSqlPassword);	/** SQL Database password */
		settings.setValue("sql_initialize", mSqlInitialize);/** SQL Initialization */
	settings.endGroup();
	
	settings.beginGroup("network");
		settings.setValue("port",			mPort);			/** port number */
		settings.setValue("backlog",		mBacklog);		/** connection backlog */
	settings.endGroup();

	settings.beginGroup("server");
		settings.setValue("server_name",	mServerName);	/** server name */
		settings.setValue("logging",		mLogging);		/** logging level */
	settings.endGroup();

	settings.beginGroup("storage");
		settings.setValue("data_root",		mDataRoot);		/** root folder for data storage */
		settings.setValue("query_root",		mQueryRoot);	/** root folder for sql query storage */
		settings.setValue("forms_root",		mFormsRoot); 	/** root folder for forms storage */
	settings.endGroup();
}

void ICEServerSettings::restore()
{
	QSettings settings(QSettings::NativeFormat, QSettings::UserScope, "pikeaero.com", "ICE Server");
	
	settings.beginGroup("sql");
		mSqlType	  = settings.value("sql_type",		"QMYSQL").toString();		/** Qt database type (e.g. "QMYSQL"). */
		mSqlServer	  = settings.value("sql_server",	"localhost").toString();	/** SQL Database server address */
		mSqlDatabase  = settings.value("sql_database",	"ice").toString();			/** SQL Database name */
		mSqlUser	  = settings.value("sql_user",		"ice").toString();			/** SQL Database user name (login) */
		mSqlPassword  = settings.value("sql_password",	"ice").toString();			/** SQL Database password */
		mSqlInitialize= settings.value("sql_initialize",false).toBool();			/** SQL Initialization */
	settings.endGroup();
	
	settings.beginGroup("network");
		mPort		= settings.value("port",	8088).toInt();					/** port number */
		mBacklog	= settings.value("backlog",	40).toInt();					/** connection backlog */
	settings.endGroup();
	
	settings.beginGroup("server");
	mServerName	= settings.value("server_name","(noname)").toString();	
	mLogging	= settings.value("logging", 9).toInt();						/** logging level */
	settings.endGroup();
	
	settings.beginGroup("storage");
		mDataRoot	= settings.value("data_root", "/opt/ice/data").toString();	/** root folder for data storage */
		mQueryRoot	= settings.value("query_root", "/opt/ice/sql").toString();	/** root folder for sql query storage */
		mFormsRoot	= settings.value("forms_root", "").toString(); 				/** root folder for forms data load */
	settings.endGroup();
}
