/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   info@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ICESERVERSETTINGS_H
#define ICESERVERSETTINGS_H

#include <QObject>
#include <QSettings>

/**
	@author R. Michael Sharkey <mike@pikeaero.com>
*/
class ICEServerSettings : public QObject
{
	Q_OBJECT
	public:
		ICEServerSettings(QObject *parent = 0);
		~ICEServerSettings();
		/** getters... */
		inline int		getPort()				{return mPort;}
		inline int		getBacklog()			{return mBacklog;}
		inline int		getLogging()			{return mLogging;}
		inline QString	getSqlType()			{return mSqlType;}
		inline QString	getSqlServer()			{return mSqlServer;}
		inline QString	getSqlDatabase()		{return mSqlDatabase;}
		inline QString	getSqlUser()			{return mSqlUser;}
		inline QString	getSqlPassword()		{return mSqlPassword;}
		inline QString	getDataRoot()			{return mDataRoot;}
		inline QString	getQueryRoot()			{return mQueryRoot;}
		inline bool		getSqlInitialize()		{return mSqlInitialize;}
		inline QString	getFormsRoot()			{return mFormsRoot;}
		inline QString	getServerName()			{return mServerName;}
		/** setters... */
		inline void setPort(int n)				{mPort=n;}
		inline void	setBacklog(int n)			{mBacklog=n;}
		inline void setLogging(int n)			{mLogging=n;}
		inline void	setSqlType(QString s)		{mSqlType=s;}
		inline void setSqlServer(QString s)		{mSqlServer=s;}
		inline void setSqlDatabase(QString s)   {mSqlDatabase=s;}
		inline void setSqlUser(QString s)		{mSqlUser=s;}
		inline void setSqlPassword(QString s)	{mSqlPassword=s;}
		inline void setDataRoot(QString s)		{mDataRoot=s;}
		inline void setQueryRoot(QString s)		{mQueryRoot=s;}
		inline void setSqlInitialize(bool s)	{mSqlInitialize=s;}
		inline void setFormsRoot(QString s)		{mFormsRoot=s;}
		inline void	setServerName(QString s)	{mServerName=s;}
		
	public slots:
		void save();
		void restore();
	
	private:
		/** ICE Server */
		int                     mPort;			/** port to listen on */
		int                     mBacklog;		/** connection backlog */
		int						mLogging;		/** logging level */
		/** SQL login */
		QString					mSqlType;		/** Qt database type (e.g. "QMYSQL"). */
		QString					mSqlServer;     /** SQL Database server address */
		QString					mSqlDatabase;	/** SQL Database name */
		QString					mSqlUser;		/** SQL Database user name (login) */
		QString					mSqlPassword;	/** SQL Database password */
		QString					mDataRoot;		/** Root folder for data storage */
		QString					mQueryRoot;		/** Root folder for sql query storage */
		bool					mSqlInitialize;	/** Initialize the SQL tables? */
		QString					mFormsRoot;		/** Directory path from which to load forms into sql table */
		QString					mServerName;	/** The "ICE" name of the server. */
};

#endif
