/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   info@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ICESERVERSTATISTICS_H
#define ICESERVERSTATISTICS_H

#include <QObject>
#include <QMutex>
#include <QMutexLocker>
#include <QDateTime>

/**
Maintain server statistics.

	@author R. Michael Sharkey <mike@pikeaero.com>
*/
class ICEServerStatistics : public QObject
{
Q_OBJECT
	public:
		ICEServerStatistics(QObject *parent = 0);
		~ICEServerStatistics();

		
	public:
			
		/**
		** Packet statistics.
		*/
		static void	setTXPackets(quint64 txb=0L);	/** set/reset the TX packet count */
		static void	setRXPackets(quint64 rxb=0L);	/** set/reset the RX packet count */
		
		static void	incrementTXPackets();			/** increment the TX packet count */
		static void	incrementRXPackets();			/** increment the RX packet count */
		
		static quint64 getTXPackets();				/** get the total TX packet count */
		static quint64 getRXPackets();				/** get the total RX packet count */
	
		/**
		** Time, dates statistics.
		*/
		static QDateTime getStartDateTime() { return mStartDateTime; }
		
	private:
		static	QMutex		mTXPacketsMutex;		/** TX Packet mutex */
		static	quint64		mTXPackets;				/** Number of TX packets */
		
		static	QMutex		mRXPacketsMutex;		/** RX Packet mutex */
		static	quint64		mRXPackets;				/** Number of RX packets */
		
		static	QDateTime	mStartDateTime;			/** Server start time. */
};

#endif
