/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   info@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "iceuserrecord.h"
#include "icesql.h"

#define inherited ICERecord

ICEUserRecord::ICEUserRecord ( ICELog* log, quint64 _id )
 : inherited ( log )
{
	load ( _id );
}

ICEUserRecord::ICEUserRecord ( ICELog* log, QString _username )
 : inherited ( log )
{
	load ( _username );
}

ICEUserRecord::ICEUserRecord ( ICELog* log )
: inherited ( log )
, m_id ( 0L )
, m_login ( "" )
{}

ICEUserRecord::~ICEUserRecord()
{}

bool ICEUserRecord::load ( quint64 _id )
{
	mValid=false;
	QSqlQuery query = ICESql::query( "user", "by_id", _id );
	if ( query.first() )
	{
		m_id = _id;
		m_login = query.value ( 0 ).toString();
		mValid = true;
	}
	return valid();
}

bool ICEUserRecord::load ( QString _login )
{
	mValid=false;
	QSqlQuery query = ICESql::query( "user", "by_name", &_login );
	if ( query.first() )
	{
		m_id = query.value ( 0 ).toUInt();
		m_login = _login;
		mValid = true;
	}
	return valid();
}


bool ICEUserRecord::new_id ( QString _login, QString _key )
{
	QSqlQuery query = ICESql::query( "user", "new", &_login, &_key );
	if ( load( _login ) ) {
		return set_firstlogin( QDateTime::currentDateTime() );
	} 
	return false;;
}

/*** setters ***/

/* set id - we don't change the user number in the record but rather, switch to a different user number */
bool ICEUserRecord::set_id ( quint64 _id )
{
	bool rc=load ( _id );
	return rc;
}

/* set login  */
bool ICEUserRecord::set_login ( QString _login )
{
	bool rc=false;
	if ( valid() )
	{
		QSqlQuery query = ICESql::query( "user", "set_login", &_login, m_id );
		rc = ( query.lastError().type() ==  QSqlError::NoError );
		if ( !rc )
		{
			emit signalError ( this, get_id(), 0,  query.lastError().driverText() );
		}
	}
	return rc;
}

/* set key  */
bool ICEUserRecord::set_key ( QString _key )
{
	bool rc=false;
	if ( valid() )
	{
		QSqlQuery query = ICESql::query( "user", "set_key", &_key, m_id );
		rc = ( query.lastError().type() ==  QSqlError::NoError );
		if ( !rc )
		{
			emit signalError ( this, get_id(), 0,  query.lastError().driverText() );
		}
	}
	return rc;
}

/* set email  */
bool ICEUserRecord::set_email ( QString _email )
{
	bool rc=false;
	if ( valid() )
	{
		QSqlQuery query = ICESql::query( "user", "set_email", &_email, m_id );
		rc = ( query.lastError().type() ==  QSqlError::NoError );
		if ( !rc )
		{
			emit signalError ( this, get_id(), 0,  query.lastError().driverText() );
		}
	}
	return rc;
}

/* set lastlogin  */
bool ICEUserRecord::set_lastlogin ( QDateTime _lastlogin )
{
	bool rc=false;
	if ( valid() )
	{
		QString lastlogin = _lastlogin.toString ( "yyyyMMddhhmmss" );
		QSqlQuery query = ICESql::query( "user", "set_lastlogin", &lastlogin, m_id );
		rc = ( query.lastError().type() ==  QSqlError::NoError );
		if ( !rc )
		{
			emit signalError ( this, get_id(), 0,  query.lastError().driverText() );
		}
	}
	return rc;
}

/* set firstlogin  */
bool ICEUserRecord::set_firstlogin ( QDateTime _firstlogin )
{
	bool rc=false;
	if ( valid() )
	{
		QString firstlogin = _firstlogin.toString ( "yyyyMMddhhmmss" );
		QSqlQuery query = ICESql::query( "user", "set_firstlogin", &firstlogin, m_id );
		rc = ( query.lastError().type() ==  QSqlError::NoError );
		if ( !rc )
		{
			emit signalError ( this, get_id(), 0,  query.lastError().driverText() );
		}
	}
	return rc;
}

/* set totalminutes  */
bool ICEUserRecord::set_totalminutes ( double _totalminutes )
{
	bool rc=false;
	if ( valid() )
	{
		QSqlQuery query = ICESql::query( "user", "set_totalminutes", _totalminutes, m_id );
		rc = ( query.lastError().type() ==  QSqlError::NoError );
		if ( !rc )
		{
			emit signalError ( this, get_id(), 0,  query.lastError().driverText() );
		}
	}
	return rc;
}

/* set totalupload  */
bool ICEUserRecord::set_totalupload ( double _totalupload )
{
	bool rc=false;
	if ( valid() )
	{
		QSqlQuery query = ICESql::query( "user", "set_totalupload", _totalupload, m_id );
		rc = ( query.lastError().type() ==  QSqlError::NoError );
		if ( !rc )
		{
			emit signalError ( this, get_id(), 0,  query.lastError().driverText() );
		}
	}
	return rc;
}

/* set totaldownload  */
bool ICEUserRecord::set_totaldownload ( double _totaldownload )
{
	bool rc=false;
	if ( valid() )
	{
		QSqlQuery query = ICESql::query( "user", "set_totaldownload", _totaldownload, m_id );
		rc = ( query.lastError().type() ==  QSqlError::NoError );
		if ( !rc )
		{
			emit signalError ( this, get_id(), 0,  query.lastError().driverText() );
		}
	}
	return rc;
}



/*** getters ***/


/* get id */
quint64  ICEUserRecord::get_id()
{
	return m_id;
}

/* get login  */
QString ICEUserRecord::get_login()
{
	return m_login;
}

/* get key  */
QString ICEUserRecord::get_key()
{
	QString rc ( "" );
	if ( valid() )
	{
		QSqlQuery query = ICESql::query( "user", "get_key", m_id );
		if ( query.first() )
		{
			rc = query.value ( 0 ).toString();
		}
		else
		{
			emit signalError ( this, get_id(),  0, query.lastError().driverText() );
		}
	}
	return rc;
}

/* get email  */
QString ICEUserRecord::get_email()
{
	QString rc ( "" );
	if ( valid() )
	{
		QSqlQuery query = ICESql::query( "user", "get_email", m_id );
		if ( query.first() )
		{
			rc = query.value ( 0 ).toString();
		}
		else
		{
			emit signalError ( this, get_id(),  0, query.lastError().driverText() );
		}
	}
	return rc;
}

/* get lastlogin  */
QDateTime ICEUserRecord::get_lastlogin()
{
	QDateTime rc;
	if ( valid() )
	{
		QSqlQuery query = ICESql::query( "user", "get_lastlogin", m_id );
		if ( query.first() )
		{
			rc = QDateTime::fromString ( query.value ( 0 ).toString(), Qt::ISODate );
		}
		else
		{
			emit signalError ( this, get_id(),  0, query.lastError().driverText() );
		}
	}
	return rc;
}

/* get firstlogin  */
QDateTime ICEUserRecord::get_firstlogin()
{
	QDateTime rc;
	if ( valid() )
	{
		QSqlQuery query = ICESql::query( "user", "get_firstlogin", m_id );
		if ( query.first() )
		{
			rc = QDateTime::fromString ( query.value ( 0 ).toString(), Qt::ISODate );
		}
		else
		{
			emit signalError ( this, get_id(),  0, query.lastError().driverText() );
		}
	}
	return rc;
}

/* get totalminutes  */
double ICEUserRecord::get_totalminutes()
{
	double rc;
	if ( valid() )
	{
		QSqlQuery query = ICESql::query( "user", "get_totalminutes", m_id );
		if ( query.first() )
		{
			rc = query.value ( 0 ).toDouble();
		}
		else
		{
			emit signalError ( this, get_id(),  0, query.lastError().driverText() );
		}
	}
	return rc;
}

/* get totalupload  */
double ICEUserRecord::get_totalupload()
{
	double rc;
	if ( valid() )
	{
		QSqlQuery query = ICESql::query( "user", "get_totalupload", m_id );
		if ( query.first() )
		{
			rc = query.value ( 0 ).toDouble();
		}
		else
		{
			emit signalError ( this, get_id(),  0, query.lastError().driverText() );
		}
	}
	return rc;
}

/* get totaldownload  */
double ICEUserRecord::get_totaldownload()
{
	double rc;
	if ( valid() )
	{
		QSqlQuery query = ICESql::query( "user", "get_totaldownload", m_id );
		if ( query.first() )
		{
			rc = query.value ( 0 ).toDouble();
		}
		else
		{
			emit signalError ( this, get_id(),  0, query.lastError().driverText() );
		}
	}
	return rc;
}

