/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   info@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ICEUSERRECORD_H
#define ICEUSERRECORD_H

#include <icerecord.h>

/**
	@author R. Michael Sharkey <mike@pikeaero.com>
*/
#include <QObject>
#include <QString>
#include <QVariant>
#include <QtSql>

#include "icerecord.h"

class ICEUserRecord : public ICERecord
{
	public:
		ICEUserRecord(ICELog* log, quint64 _id);
		ICEUserRecord(ICELog* log, QString _username);
		ICEUserRecord(ICELog* log=NULL);
		~ICEUserRecord();
		/** load by user number */
		virtual bool          load(quint64 _id);
		/** load by login */
		virtual bool          load(QString _login);
		/** record is valid? */
		virtual bool          valid() {return mValid;}
		/** create a new record/id */
		virtual bool            new_id(QString _login, QString _key);

		/*** setters ***/

		/** set id - we don't change the user number in the record but rather, switch to a different user number */
		virtual bool            set_id(quint64 _id);
		/** set login  */
		virtual bool            set_login(QString _login);
		/** set key  */
		virtual bool            set_key(QString _key);
		/** set email  */
		virtual bool            set_email(QString _email);
		/** set lastlogin  */
		virtual bool            set_lastlogin(QDateTime _lastlogin);
		/** set first_first  */
		virtual bool            set_firstlogin(QDateTime _firstlogin);
		/** set totalminutes */
		virtual bool            set_totalminutes(double _totalminutes);
		/** set totalupload */
		virtual bool            set_totalupload(double _totalupload);
		/** set totaldownload */
		virtual bool            set_totaldownload(double _totaldownload);
		/*** getters ***/

		/** get id */
		virtual quint64   		get_id();
		/** get login  */
		virtual QString         get_login();
		/** get key  */
		virtual QString         get_key();
		/** get email  */
		virtual QString         get_email();
		/** get lastlogin  */
		virtual QDateTime       get_lastlogin();
		/** get firstlogin  */
		virtual QDateTime       get_firstlogin();
		/** get totalminutes */
		virtual double          get_totalminutes();
		/** get totalupload */
		virtual double          get_totalupload();
		/** get totaldownload */
		virtual double          get_totaldownload();

	private:
		/* record is valid? */
		bool          mValid;
		/* field: id */
		quint64 m_id;
		/* field: login */
		QString       m_login;
};


#endif
