/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   info@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ICESERVERSESSION_H
#define ICESERVERSESSION_H

#include <QObject>
#include <QTcpSocket>
#include <QThread>
#include <QTimerEvent>
#include <QSqlDatabase>
#include <QList>
#include <QFile>
#include <QMutex>
#include <QMutexLocker>
    
#include "icesession.h"
#include "icedatabase.h"
#include "iceuserrecord.h"
#include "icelog.h"
#include "icesessionobject.h"

/**
@author Michael Sharkey <mike@pikeaero.com>
*/
class ICEServerSession : public ICESession
{
	Q_OBJECT
	public:

		ICEServerSession(QObject *parent = 0, QTcpSocket* socket = 0, ICEDatabase* database=0 );
		~ICEServerSession();

		virtual		ICESessionObject* construct(ICESessionObject::ObjectType,QByteArray* id=NULL,QByteArray* data=NULL);

		ICEUserRecord*			user()			{return &mUser;}

	public slots:
		virtual     void        asyncPopupMessage(QString text,ICEPacket::PacketCode code=ICEPacket::CodeOK);
		virtual     void        rx(ICEPacket rxpacket);           /* receive a packet */
		
	protected:
		virtual     void		tx(ICEPacket& packet);
		virtual		void		timerEvent(QTimerEvent* e);
		virtual     int         generateRandomNumber(int lo, int hi);
		virtual     QByteArray  generateChallenge();

		virtual     void        replyDisconnect(ICEPacket*,ICEPacket::PacketCode code=ICEPacket::CodeOK);
		virtual     void        replyConnection(ICEPacket*,ICEPacket::PacketCode code=ICEPacket::CodeOK);
		virtual     void        replyConnectNewLoginID(ICEPacket*,ICEPacket::PacketCode code=ICEPacket::CodeOK);
		virtual     void        replyLogin(ICEPacket*,ICEPacket::PacketCode code=ICEPacket::CodeOK);
		virtual     void        replyDeleteLogin(ICEPacket*,ICEPacket::PacketCode code=ICEPacket::CodeOK);
		virtual     void        replySession(ICEPacket*,ICEPacket::PacketCode code=ICEPacket::CodeOK);
		virtual		void		replySessionObjectConstruct(ICEPacket*,ICEPacket::PacketCode code=ICEPacket::CodeOK);
		
		virtual		void		processReplySessionObjectUi(ICEPacket*);
	protected slots:
		virtual     void        slotConnected();
		virtual     void        slotDisconnected();
		virtual     void        slotError(QAbstractSocket::SocketError err);
		virtual     void        slotHostFound();
		virtual     void        slotStateChanged(QAbstractSocket::SocketState state);

	private:
		QByteArray              mChallenge;                     /** login challenge data */
		ICEDatabase*            mDatabase;						/** The database in use */
		ICEUserRecord           mUser;
		ICELog                  mLog;
		int                     mCipherAttempts;                /** # attempts to agree on cipher */
		ICEPacket::PlatformType mPlatform;
		QList<QFile *>          mOpenFiles;
		int						mMinuteTimer;
		static QMutex			mRandMutex;
};

#endif
