/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   info@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "iceserver.h"
#include "icesession.h"
#include "iceserversettings.h"
#include "iceserverstatistics.h"
#include "iceserversessionobjectuitransport.h"
#include "iceserverscriptthread.h"

#include <QFile>
#include <QFileInfo>
#include <QUrl>

extern ICEServerSettings serverSettings; /** FIXME - should be a static member of ICEServer */

#define inherited ICESessionObject

ICEServerSessionObjectUiTransport::ICEServerSessionObjectUiTransport(ICEServerSession *session,QByteArray* id)
 : inherited(session,id)
{
}

ICEServerSessionObjectUiTransport::~ICEServerSessionObjectUiTransport()
{
}

QString ICEServerSessionObjectUiTransport::location()
{
	return mLocation;
}

void ICEServerSessionObjectUiTransport::setUi(QByteArray ui)
{
	/** The form XML data */	
	mUi=ui;
	session()->requestSessionObjectUi( this );
}

ICEScriptThread*  ICEServerSessionObjectUiTransport::setLocation(QString location)
{
	ICEScriptThread* script=NULL;
	QString error;
	QUrl url ( (mLocation = location ) );
	QString localPath = serverSettings.getDataRoot().trimmed() + url.path().trimmed();
	QFileInfo fi( localPath );
	if ( fi.isDir() )
	{
		if ( localPath.right(1) != "/" )
		{
			localPath += "/";
		}
		localPath += "ice.js";
	}
	fi = QFileInfo( localPath );
	if ( fi.isFile() && fi.isReadable() )
	{
		if ( fi.size() <= 32*1024 )	/** FIXME - make a max size setting per session and per session object */
		{
			emit statusMessage( this, "loading: "+localPath );
			QFile iceFile( localPath );
			if ( iceFile.open( QIODevice::ReadOnly ) )
			{
				script = new ICEServerScriptThread( this );
				if ( script != NULL )
				{
					script->load( iceFile.readAll() );
				}
			}
			else
			{
				error = "failed to open: "+localPath;
			}
		}
		else
		{
			error = "too big: "+localPath;
		}
	}
	else
	{
		error = "not found or not readable: "+localPath;
	}
	if ( script != NULL )
	{
		mScripts.append(script);
	}
	if ( !error.isEmpty() )
	{
		/** FIXME - return error code to client */
		emit statusMessage( this, error );
		session()->asyncPopupMessage( error );
		destroy();
		script=NULL;
	}
	return script;
}

