/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   info@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "iceui.h"
#include "icesession.h"
#include "icesessionobject.h"

#include <QFileInfo>
#include <QFile>

#define inherited QObject

ICEUi::ICEUi(ICEScriptThread *parent)
: inherited()
, mUiRootObject(NULL)
, mScriptThread(parent)
{
	QObject::connect(this,SIGNAL(uiData(QByteArray)),parent->sessionObject(),SLOT(setUi(QByteArray)));
}


ICEUi::~ICEUi()
{
}

void ICEUi::load( QString ui )
{
	bool rc=false;
	QFileInfo fi( ui );
	QString error;
	if ( fi.isFile() )
	{
		if ( fi.isReadable() )
		{
			QFile f( ui );
			if ( f.open( QIODevice::ReadOnly ) )
			{
				load( f.readAll() );
				rc=true;
			}
			else {
				error =  "open failed: \""+ui+"\"";
			}
		}
		else
		{
			error = "ui is not readable: \""+ui+"\"";
		}
	}
	else
	{
		error = "ui is not a regular file: \""+ui+"\"";
	}
	if ( rc )
	{
		emit loaded();
	}
	else
	{
		emit loadFailed( error );
	}
}

void ICEUi::load( QByteArray ui )
{
	emit uiData( ui );
}

/**
** Return the path to an UI object as "/root/.../object"
*/
QString ICEUi::objectPath( QObject* o )
{
	QString s;
	if ( o != NULL ) 
	{
		do
		{
			s = QString("/") + o->objectName() + s;
			o = o->parent();
		}
		while ( o != NULL );
	}
	return s;
}

/**
** Interpret a path to a UI object as "/root/.../object" and return the matching object or NULL.
*/
ICEUiElement* ICEUi::object( QString path )
{
	path = path.trimmed();
	for(int i=0;i<mUiObjects.count();i++)
	{
		if ( path == objectPath( mUiObjects[i] ) )
		{
			return mUiObjects[i];
		}
	}
	return NULL;
}

bool ICEUi::eventFilter(QObject *watched, QEvent *event)
{
	switch( event->type() )
	{
		/** FIXME - trap events here */
	}
	return inherited::eventFilter( watched, event );
}

qint32 ICEUi::objectId( QString path )
{
	for(qint32 i=0;i<mUiObjects.count();i++)
	{
		if ( path == objectPath( mUiObjects[i] ) )
		{
			return i;
		}
	}
	return -1;
}

ICESession* ICEUi::session()
{
	return script()->session();
}

ICESessionObject* ICEUi::sessionObject()
{
	fprintf( stderr, "ICEUi - FIXME!! return session object ID NOT session object ptr!!!\n" );
	return script()->sessionObject();
}

ICEScriptThread* ICEUi::script()
{
	return mScriptThread;
}