/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   info@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ICEUI_H
#define ICEUI_H

#include "iceuielement.h"

#include <QObject>
#include <QScriptable>
#include <QString>
#include <QByteArray>
#include <QMutex>

class ICESession;
class ICESessionObject;
class ICEScriptThread;

/**
ICE Ui object base class

	@author R. Michael Sharkey <mike@pikeaero.com>
*/
class ICEUi : public QObject, protected QScriptable
{
	Q_OBJECT
	public:
		ICEUi(ICEScriptThread* parent);
		~ICEUi();
	public Q_SLOTS:
		virtual	void					load( QString ui );
		virtual	void					load( QByteArray ui );
		virtual	ICEUiElement*			object( QString path );
		virtual	qint32					objectId( QString path );
		virtual	QList<ICEUiElement*>	objects() {return mUiObjects;}
		virtual	QString					objectPath( QObject* o );
		virtual	ICESession*				session();
		virtual	ICESessionObject*		sessionObject();
		virtual	ICEScriptThread*		script();
	signals:
				void					uiData( QByteArray ui_data );
				void					loadFailed( QString );
				void					loaded();
	protected:
		virtual	bool 					eventFilter(QObject *watched, QEvent *event);
				ICEUiElement*			mUiRootObject;
				QList<ICEUiElement*>	mUiObjects;
				QMutex					mUiObjectMutex;
				ICEScriptThread*		mScriptThread;
};

#endif
