/***************************************************************************
 *   Copyright (C) 2007 by Pike Aerospace Research Corporation             *
 *   info@pikeaero.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "iceuireader.h"

ICEUiReader::ICEUiReader(QByteArray & data, QObject *parent)
 : QObject( parent )
 , QXmlStreamReader( data )
{
	setNamespaceProcessing(false);
}


ICEUiReader::~ICEUiReader()
{
}


void ICEUiReader::read()
{
	while( !atEnd() )
	{
		QXmlStreamReader::TokenType t = readNext();
		switch( t )
		{
			case QXmlStreamReader::NoToken:					break;
			case QXmlStreamReader::Invalid:					break;
			case QXmlStreamReader::StartDocument:			documentStart();	break;
			case QXmlStreamReader::EndDocument:				documentEnd();		break;
			case QXmlStreamReader::StartElement:			elementStart();		break;
			case QXmlStreamReader::EndElement:				elementEnd();		break;
			case QXmlStreamReader::Characters:				characters();		break;
			case QXmlStreamReader::Comment:					break;
			case QXmlStreamReader::DTD:						break;
			case QXmlStreamReader::EntityReference:			break;
			case QXmlStreamReader::ProcessingInstruction:	break;
		}
	}
	if ( hasError() )
	{
		emit errorMessage( errorString() );
	}
}


void ICEUiReader::documentStart()
{
	//fprintf( stderr, "Ui Reader: Document Start\n" );
}

void ICEUiReader::documentEnd()
{
	//fprintf( stderr, "Ui Reader: Document End\n" );
}

void ICEUiReader::elementStart()
{
	QString element=name().toString();
	blanks();
	mElementStack.push(element);
	//fprintf( stderr, "<%s>\n", element.toAscii().data() );
	for(int i=0; i < attributes().count(); i++)
	{
		QXmlStreamAttribute a = attributes()[i];
		QString name = a.name().toString();
		QString value = a.value().toString();
		blanks();
		//fprintf( stderr, " %s=%s\n",name.toAscii().data(), value.toAscii().data() );
	}
}

void ICEUiReader::elementEnd()
{
	if ( mElementStack.count() > 0 )
		mElementStack.pop();
	QString element=name().toString();
	blanks();
	//fprintf( stderr, "</%s>\n", element.toAscii().data() );
}

void ICEUiReader::characters()
{
	QString ch = text().toString();
	blanks();
	//fprintf( stderr, "  \"%s\"\n", ch.toAscii().data() );
}

void ICEUiReader::blanks()
{
	for(int i=0; i<mElementStack.count(); i++ )
	{
		//fprintf( stderr, " " );
	}
}


